#Title:          Summary.py
#Author:         Stephen Tanner, Samuel Payne, Natalie Castellana, Pavel Pevzner, Vineet Bafna
#Created:        2005
# Copyright 2007,2008,2009 The Regents of the University of California
# All Rights Reserved
#
# Permission to use, copy, modify and distribute any part of this
# program for educational, research and non-profit purposes, by non-profit
# institutions only, without fee, and without a written agreement is hereby
# granted, provided that the above copyright notice, this paragraph and
# the following three paragraphs appear in all copies.
#
# Those desiring to incorporate this work into commercial
# products or use for commercial purposes should contact the Technology
# Transfer & Intellectual Property Services, University of California,
# San Diego, 9500 Gilman Drive, Mail Code 0910, La Jolla, CA 92093-0910,
# Ph: (858) 534-5815, FAX: (858) 534-7345, E-MAIL:invent@ucsd.edu.
#
# IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
# FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
# INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
# OF SUCH DAMAGE.
#
# THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
# OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
# ENHANCEMENTS, OR MODIFICATIONS.  THE UNIVERSITY OF CALIFORNIA MAKES NO
# REPRESENTATIONS AND EXTENDS NO WARRANTIES OF ANY KIND, EITHER IMPLIED OR
# EXPRESS, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, OR THAT THE USE OF
# THE SOFTWARE WILL NOT INFRINGE ANY PATENT, TRADEMARK OR OTHER RIGHTS.


"""
The new Summary script, modified for lower memory usage.  This version
assumes that the top annotation is the correct one, and does not consider
re-assigning a spectrum to a close runner-up or to a homologous protein.

- Iterate over annotations.  Filter any that aren't a top-scoring annotation with
good p-value.
- Remember the best modified and the best unmodified annotation for each database interval
- Generate the protein report: one sub-report for each protein, sorted by coverage
- Sub-report includes a table of covered residues, followed by a list of peptide rows.
- Each peptide row gives a spectrum count and score for the best spectrum
- Peptide rows are sorted by protein position, then mass
"""
import os
import time
import sys
import struct
import traceback
import shutil
import getopt
import Label
import MakeImage
import MSSpectrum
import SelectProteins
import ResultsParser
from Utils import *
Initialize()

UsageInfo = """Summary.py - Parse search results, and generate either a webpage
  summarizing the results, or a filtered database for unrestrictive PTM search.

Required options:
 -r [FileName] - The name of the results file to parse.  If a directory is
    specified, then all .txt files within the directory will be combined into
    one report
 -d [FileName] - The name of the database file (.trie format) searched.
         (allows more than one database file; use multiple -d options)
 
Additional options:
 -b [FileName] - Second-pass database filename.  If specified, the proteins
    selected will be written out to a database (.fasta, .trie and .index files)
    suitable for unrestrictive search.
 -w [FileName] - Webpage directory.  If specified, a webpage will be written
    to the specified directory, summarizing the proteins identified, and the
    degree of coverage.

 (Note: Either -b or -w, or both, must be provided)
    
 -p [Value] - Cutoff p-value.  Annotations with inferior p-value are ignored.
    Defaults to 0.05.
 -e [Count] - Minimum number of peptides that a protein must annotate in order
    to add it to the report or the filtered database.  Defaults to 1.
 -m [Count] - Minimum number of spectra that a protein must annotate in order
    to add it to the report or the filtered database.  By default, this count
    is set to (SpectrumCount / ProteinsInDatabase) * 2.  If the protein
    database has already been filtered, set this parameter to 1.
 -v [Count] - Verbose spectrum output count.  If set, report [Count] spectra
    for each distinct peptide identified.  This option is slower and
    consumes more memory, but can be more informative.
 -i [SpectraPath] - For use if verbose spectrum output (-v) is enabled.
    Images will be generated for each annotation, if the Python Imaging
    Library (PIL) is installed.  This option generates many files on disk,
    so it's recommended that you set the summary file (-w option) in its own
    directory.  "SpectraPath" is the path to the folder with MS2 spectra
    
    
Examples:
    Summary.py -r Frac1Ouput.txt -d Database%ssprot.trie -p 0.1
    Summary.py -r Frac1Ouput.txt -d Database%ssprot.trie -w F1Summary\index.html -v
"""%(os.sep, os.sep)

class SummarizerClass(ResultsParser.ResultsParser):
    def __init__(self):
        # SharedProteinSpectra[ProteinA][ProteinB] = number of spectra for
        # which the annotation is found in both protein A and protein B.  When
        # we accept one of the two proteins, the other one loses some annotations!
        self.SharedProteinSpectra = {}
        # SharedProteinPeptides is similar to SharedProteinSpectra, but tracks
        # distinct peptide records
        self.SharedProteinPeptides = {} 
        self.AnnotationCounts = {} # ProteinID -> count
        # BestRepresentatives[(DBStart, DBEnd)] is a list of peptide
        # instances for the best spectra for that positon.
        self.BestRepresentatives = {}
        self.BestModRepresentatives = {}
        self.ResultsFileName = None
        self.DatabasePath = []
        self.SecondPassDatabasePath = None
        self.MinimumProteinHits = None
        self.SummaryPagePath = None
        self.MZXMLPath = None
        self.PValueCutoff = 0.05
        # Keys are peptides (after I->L substitution), values are lists of protein
        # record numbers.  We keep this dictionary so that we needn't repeatedly map
        # the same protein to the database.
        self.PeptideDict = {}
        self.VerboseProteinReportCount = 0
        self.GenerateSpectrumImagesFlag = 0
        # Very short peptides are uninformative...skip them.
        self.MinimumPeptideLength = 7
        self.SpectrumCount = 0
        self.IntervalHitCount = {}
        self.MinimumPeptides = 1
        ResultsParser.ResultsParser.__init__(self)
    def GetSpectrumPath(self, Path, SpectraPath):
        """
        This requires a bit of trickery, because sometimes the results are
        generated on a unix machine (creating a unix path), and this script
        is run on a windows machine (can't split a unix path). So I have going
        to hack things out on my own.
        """
        FileName = None
        if Path.find("/") >= 0:
            "results files made on a unix machine"
            LastSlash = Path.rfind("/")
            FileName = Path[LastSlash+1:]
        else:
            "results files made on a windows machine, hopefully.  Any other users, go home"
            LastBackSlash = Path.rfind("\\")
            FileName = Path[LastBackSlash+1:]
        if not FileName:
            print "unable to create a path to the spectrum file %s"%Path
            return Path
        return os.path.join(SpectraPath,FileName)
    def WriteSecondPassDatabase(self):
        """
        Write out the "present" proteins to our second-pass database.
        self.ProteinSelector is responsible for deciding which peptides
        belong to which proteins.
        """
        Bits = os.path.split(self.SecondPassDatabasePath)
        DBPathStub = os.path.splitext(self.SecondPassDatabasePath)[0]
        if len(Bits[0]) == 0:
            DBPath = os.path.join("Database", "%s.trie"%DBPathStub)
            IndexPath = os.path.join("Database", "%s.index"%DBPathStub)
            FastaPath = os.path.join("Database", "%s.fasta"%DBPathStub)
        else:
            DBPath = DBPathStub + ".trie"
            IndexPath = DBPathStub + ".index"
            FastaPath = DBPathStub + ".fasta"
        print "Writing second-pass database to %s..."%DBPath
        DBFile = open(DBPath, "wb")
        IndexFile = open(IndexPath, "wb")
        FASTAFile = open(FastaPath, "wb")
        DBFilePos = 0
        for (ProteinID, ScoreTuple) in self.ProteinSelector.SelectedProteins.items():
            (PeptideCount, SpectrumCount) = ScoreTuple
            if SpectrumCount < self.MinimumProteinHits:
                continue
            if PeptideCount < self.MinimumPeptides:
                continue
            # Let's write out the protein.  Write to the INDEX file, the
            # TRIE file, and a FASTA file.  (The FASTA file is just for
            # humans to read)
            ProteinName = self.ProteinSelector.ProteinNames[ProteinID]
            ProteinSequence = self.ProteinSelector.ProteinSequences[ProteinID]
            Str = struct.pack("<qi80s", 0, DBFilePos, ProteinName[:80])
            IndexFile.write(Str)
            DBFile.write("%s*"%ProteinSequence)
            DBFilePos += len(ProteinSequence) + 1
            Pos = 0
            FASTAFile.write(">%s\n"%ProteinName)
            while Pos < len(ProteinSequence):
                Chunk = ProteinSequence[Pos:Pos+70]
                FASTAFile.write(Chunk)
                FASTAFile.write("\n")
                Pos += 70
        IndexFile.close()
        FASTAFile.close()
        DBFile.close()
    def GetProteinHREF(self, ProteinID, ProteinName):
        # By default, just print the name with no hyperlinking.
        # Subclass can override to hyperlink to IPI, swiss-prot, etc
        return ProteinName
    def WriteSummaryPage(self):
        """
        Produce the protein report.  The index file contains the protein coverage
        information.  If verbose output is requested, also contains one row per peptide.
        """
        Dir = os.path.split(self.SummaryPagePath)[0]
        try:
            os.makedirs(Dir)
        except:
            pass
        # Populate PeptidesForProtein.  Keys are protein IDs.  Values are lists
        # of peptide annotations; the annotations are, in turn, keys for
        # self.ProteinSelector.BestRepresentatives
        self.PeptidesForProtein = {}
        for (Annotation, RepList) in self.ProteinSelector.BestRepresentatives.items():
            if len(RepList) < 1:
                continue
            Peptide = RepList[0][1]
            ProteinID = self.ProteinSelector.PeptideProteins.get(Peptide.Aminos, None)
            if ProteinID == None:
                continue
            PeptideList = self.PeptidesForProtein.get(ProteinID, [])
            PeptideList.append(Annotation)
            self.PeptidesForProtein[ProteinID] = PeptideList
        # Sort proteins from "best" to "worst".  For now, just sort by the
        # number of distinct peptides.
        SortedProteins = []
        for (ProteinID, AnnotationList) in self.PeptidesForProtein.items():
            AnnotationCount = len(AnnotationList)
            if AnnotationCount >= self.MinimumPeptides:
                SortedProteins.append((AnnotationCount, ProteinID))
        SortedProteins.sort()
        SortedProteins.reverse()
        # Start the index file:
        self.SummaryPageDir = os.path.split(self.SummaryPagePath)[0]
        # Ensure the directory exists:
        try:
            os.makedirs(self.SummaryPageDir)
        except:
            pass
        self.IndexFile = open(self.SummaryPagePath, "w")
        self.IndexFile.write("<html><title>Protein Report</title>\n")
        # Iterate over proteins, writing one record for each one:
        for (PeptideCount, ProteinID) in SortedProteins:
            ProteinName = self.ProteinSelector.ProteinNames[ProteinID]
            if ProteinName[:3]== "XXX":
                #print "Found a fake protein %s"%ProteinName
                continue
            #print "Write protein %s (%s), %s peptides"%(ProteinID, self.ProteinSelector.ProteinNames[ProteinID], PeptideCount)
            self.WriteProteinSummary(ProteinID)
        self.IndexFile.close()
    def WriteProteinSummary(self, ProteinID):
        """
        Write summary page section for a single protein.
        """
        ##########################################################
        # Determine coverage, and sort peptides by position within the protein:
        ProteinName = self.ProteinSelector.ProteinNames[ProteinID]
        ProteinSequence = self.ProteinSelector.ProteinSequences[ProteinID]
        Coverage = [0] * len(ProteinSequence)
        PeptideCount = len(self.PeptidesForProtein.get(ProteinID, []))
        for Annotation in self.PeptidesForProtein.get(ProteinID, []):
            Peptide = self.ProteinSelector.BestRepresentatives[Annotation][0][1]
            MatchPos = ProteinSequence.find(Peptide.Aminos)
            if MatchPos == -1:
                print "** Error: Peptide '%s' assigned to incompatible protein %s '%s'"%(Peptide.Aminos, ProteinID, ProteinName)
            for Pos in range(MatchPos, MatchPos + len(Peptide.Aminos)):
                Coverage[Pos] += 1
        CoverFlags = 0
        for CoverageCount in Coverage:
            if CoverageCount:
                CoverFlags += 1
        CoverageRate = CoverFlags / float(len(ProteinSequence))
        # Write header:
        SpectrumCount = self.ProteinSelector.ProteinSpectrumCounts[ProteinID]
        HREF = self.GetProteinHREF(ProteinID, ProteinName)
        self.IndexFile.write("<h3>%s</h3>\n<b>%s peptides, %s spectra, %.1f%% coverage</b><br>\n"%(HREF, PeptideCount, SpectrumCount, CoverageRate*100))
        # Write protein sequence:
        ColorUncovered = "#aaaaaa"
        ColorCovered = "#000000"
        OldColor = ColorUncovered
        OldBoldFlag = 0
        BoldFlag = 0
        self.IndexFile.write("<tt>")
        for Pos in range(len(ProteinSequence)):
            ResidueNumber = Pos + 1
            if ResidueNumber%50 == 1:
                if BoldFlag:
                    self.IndexFile.write("</b>")
                    BoldFlag = 0                  
                self.IndexFile.write("</font><br>\n<font color=#000000>")
                OldColor = ColorCovered
                OldBoldFlag = 0
                for Foo in range(4 - len(str(ResidueNumber))):
                    self.IndexFile.write("&nbsp;")
                self.IndexFile.write("%d "%ResidueNumber)
            if ResidueNumber % 10 == 1:
                self.IndexFile.write(" ")
            if Coverage[Pos]:
                Color = ColorCovered
            else:
                Color = ColorUncovered
                BoldFlag = 0
            if Color != OldColor:
                self.IndexFile.write("</font><font color=%s>"%Color)
                OldColor = Color
            if BoldFlag != OldBoldFlag:
                if BoldFlag:
                    self.IndexFile.write("<b>")
                else:
                    self.IndexFile.write("</b>")
                OldBoldFlag = BoldFlag
            self.IndexFile.write("%s"%ProteinSequence[Pos])
        self.IndexFile.write("<br><br></font><font color=#000000></tt>\n\n")
        ###############################################
        # Write individual peptides, if requested:
        if self.VerboseProteinReportCount:
            # Write out peptides:
            self.WritePeptideHeader(ProteinID, self.IndexFile)
            SortedAnnotations = []
            for Annotation in self.PeptidesForProtein.get(ProteinID, []):
                RepresentativeList = self.ProteinSelector.BestRepresentatives[Annotation]
                Peptide = RepresentativeList[0][1]
                Pos = ProteinSequence.find(Peptide.Aminos)
                SortedAnnotations.append((Pos, Pos + len(Peptide.Aminos) - 1, Annotation))
            SortedAnnotations.sort()
            for (StartPos, EndPos, Annotation) in SortedAnnotations:
                IntervalString = "%s-%s"%(StartPos + 1, EndPos + 1)
                TotalHitCount = self.ProteinSelector.AnnotationSpectrumCounts[Annotation]
                RepresentativeList = self.ProteinSelector.BestRepresentatives[Annotation]
                RepresentativeList.reverse() # they're sorted from worst-to-best; fix that.
                for Index in range(len(RepresentativeList)):
                    Peptide = RepresentativeList[Index][-1]
                    self.WritePeptideLine(self.IndexFile, IntervalString, ProteinID, Index, Peptide, TotalHitCount)
            self.WritePeptideFooter(self.IndexFile)
        self.IndexFile.write("<hr>")
    def WritePeptideFooter(self, IndexFile):
        IndexFile.write("</table>\n")
    def WritePeptideHeader(self, ProteinID, IndexFile):
        IndexFile.write("<table><tr><td><b>Residues</b></td><td><b>Total Spectra</b></td><td><b>Peptide</b></td><td><b>p-value</b></td><td><b>MQScore</b></td><td><b>File</b></td><td><b>Scan</b></td></tr>")
    def WritePeptideLine(self, File, IntervalStr, ProteinID, SpectrumIndex, Peptide, TotalHitCount):
        Dir = os.path.split(self.SummaryPagePath)[0]
        Annotation = Peptide.GetModdedName()
        SpecFileName = Peptide.SpectrumFilePath.replace("/","\\").split("\\")[-1]
        if self.GenerateSpectrumImagesFlag:
            ImageFileName = "%s.%s.png"%(Annotation, SpectrumIndex)
            ImageFilePath = os.path.join(Dir, ImageFileName)
            Maker = MakeImage.MSImageMaker()
            MSpectrum = MSSpectrum.SpectrumClass()
            Path = self.GetSpectrumPath(Peptide.SpectrumFilePath, self.MZXMLPath)
            FileName = "%s:%s"%(Path, Peptide.SpectrumFilePos)
            try:
                #SpectrumFile = Label.OpenAndSeekFile(FileName)
                #print FileName
                #MSpectrum.ReadPeaksFromFile(SpectrumFile, FileName)
                #MSpectrum.RankPeaksByIntensity()
                #SpectrumFile.close()
                #Label.LabelSpectrum(MSpectrum, Peptide)
                #Maker.ConvertSpectrumToImage(MSpectrum, ImageFilePath, Peptide, Width = 500, Height = 380)
                Args = " -r %s -b %d -a %s -w %s -p"%(Path, int(Peptide.SpectrumFilePos), Annotation, ImageFilePath)
                ArgsList = Args.split()
                #print "Parsing Results for %s, scan %s"%(FileName, Scan)
                Dymo = Label.LabelClass()
                Dymo.ParseCommandLine(ArgsList)
                #Dymo.LoadModel(0, Dymo.PeptideHasPhosphorylation)
                Dymo.Main()
                
            except:
                traceback.print_exc()
            File.write("<tr><td>%s</td><td>%s</td><td><a href=\"%s\">%s</td><td>%s</td><td>%s</td>"%(IntervalStr, TotalHitCount, ImageFileName, Peptide.GetFullModdedName(), Peptide.PValue, Peptide.MQScore))
            File.write("<td>%s</td><td>%s</td></tr>\n"%(SpecFileName, Peptide.ScanNumber))
        else:
            File.write("<tr><td>%s</td><td>%s</td><td>%s</td><td>%s</td><td>%s</td>"%(IntervalStr, TotalHitCount, Peptide.GetFullModdedName(), Peptide.PValue, Peptide.MQScore))
            File.write("<td>%s</td><td>%s</td></tr>\n"%(SpecFileName, Peptide.ScanNumber))
    def SetMinimumProteinHits(self):
        ProteinCount = len(self.ProteinNames)
        self.MinimumProteinHits = (2 * self.SpectrumCount) / ProteinCount
        self.MinimumProteinHits = max(self.MinimumProteinHits, 2)
        print "%s spectra with a valid annotation, %s proteins in all"%(self.SpectrumCount, ProteinCount)
        print "Minimum hits required to accept an additional protein: ", self.MinimumProteinHits
    def Main(self):
        self.ProteinSelector = SelectProteins.ProteinSelector()
        print self.PValueCutoff
        self.ProteinSelector.PValueCutoff = self.PValueCutoff
        self.ProteinSelector.LoadMultipleDB(self.DatabasePath)
        # If we're expected to write out a summary page, then keep track
        # of the top N representatives for each annotation:
        if self.SummaryPagePath:
            if self.VerboseProteinReportCount:
                self.ProteinSelector.RetainRepresentativeCount = self.VerboseProteinReportCount
            else:
                self.ProteinSelector.RetainRepresentativeCount = 1
        self.ProcessResultsFiles(self.ResultsFileName, self.ProteinSelector.ParseAnnotations)
        self.ProteinSelector.ChooseProteins()
        if self.SecondPassDatabasePath:
            self.WriteSecondPassDatabase()
            print "Second-pass database written to:", self.SecondPassDatabasePath
        if self.SummaryPagePath:
            self.WriteSummaryPage()
            print "Summary page written to:", self.SummaryPagePath
    def ParseCommandLine(self, Arguments):
        (Options, Args) = getopt.getopt(Arguments, "r:d:b:p:w:m:v:i:e:")
        OptionsSeen = {}
        for (Option, Value) in Options:
            OptionsSeen[Option] = 1
            if Option == "-r":
                # -r results file(s)
                if not os.path.exists(Value):
                    print "** Error: couldn't find results file '%s'\n\n"%Value
                    print UsageInfo
                    sys.exit(1)
                self.ResultsFileName = Value
            elif Option == "-d":
                # -d database
                if not os.path.exists(Value):
                    print "** Error: couldn't find database file '%s'\n\n"%Value
                    print UsageInfo
                    sys.exit(1)
                self.DatabasePath.append(Value)
            elif Option == "-b":
                # -b Second-pass database
                self.SecondPassDatabasePath = Value
            elif Option == "-i":
                self.GenerateSpectrumImagesFlag = 1
                self.MZXMLPath = Value
            elif Option == "-m":
                # -m Minimum number of spectra for a new protein
                self.MinimumProteinHits = int(Value)
            elif Option == "-e":
                # -e Minimum number of peptides for a new protein
                self.MinimumPeptides = int(Value)
            elif Option == "-w":
                # -w Summary page filename
                self.SummaryPagePath = Value
            elif Option == "-p":
                # -p p-value cutoff
                self.PValueCutoff = float(Value)
                print self.PValueCutoff
            elif Option == "-v":
                # -v Verbose output flag
                self.VerboseProteinReportCount = int(Value)
        # Error out, if we didn't see required options:
        if not OptionsSeen.has_key("-d") or not OptionsSeen.has_key("-r"):
            print "** Please specify database (-d) and results file (-r)"
            print UsageInfo
            sys.exit(1)
        # If neither -b nor -w was specified, assume they want a summary:
        if not OptionsSeen.has_key("-b") and not OptionsSeen.has_key("-w"):
            self.SummaryPagePath = os.path.join("ProteinSummary", "index.html")
            print "** Summary page will be written to '%s'; use -w to override this"%Summarizer.SummaryPagePath
        print "Summary page path:", self.SummaryPagePath
        
if __name__ == "__main__":
    try:
        import psyco
        psyco.full()
    except:
        print "(psyco not found - running in non-optimized mode)"
    Summarizer = SummarizerClass()
    Summarizer.ParseCommandLine(sys.argv[1:])
    StartTime = time.clock()
    Summarizer.Main()
    EndTime = time.clock()
    print "ELAPSED:", EndTime - StartTime



#Title:          LDA.py
#Author:         Stephen Tanner, Samuel Payne, Natalie Castellana, Pavel Pevzner, Vineet Bafna
#Created:        2005
# Copyright 2007,2008,2009 The Regents of the University of California
# All Rights Reserved
#
# Permission to use, copy, modify and distribute any part of this
# program for educational, research and non-profit purposes, by non-profit
# institutions only, without fee, and without a written agreement is hereby
# granted, provided that the above copyright notice, this paragraph and
# the following three paragraphs appear in all copies.
#
# Those desiring to incorporate this work into commercial
# products or use for commercial purposes should contact the Technology
# Transfer & Intellectual Property Services, University of California,
# San Diego, 9500 Gilman Drive, Mail Code 0910, La Jolla, CA 92093-0910,
# Ph: (858) 534-5815, FAX: (858) 534-7345, E-MAIL:invent@ucsd.edu.
#
# IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
# FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
# INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
# OF SUCH DAMAGE.
#
# THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
# OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
# ENHANCEMENTS, OR MODIFICATIONS.  THE UNIVERSITY OF CALIFORNIA MAKES NO
# REPRESENTATIONS AND EXTENDS NO WARRANTIES OF ANY KIND, EITHER IMPLIED OR
# EXPRESS, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, OR THAT THE USE OF
# THE SOFTWARE WILL NOT INFRINGE ANY PATENT, TRADEMARK OR OTHER RIGHTS.


"""
Linear discriminant analysis
Assumes the input file is tab-delimited, with category in the first column, and float
values in the remaining columns.
"""

USE_NUMPY = 1
import traceback
try:
    if USE_NUMPY:
        from numpy import *
        import numpy.linalg
        MatrixMulitply = dot
        InvertMatrix = numpy.linalg.inv
        FloatType = float
    else:
        from Numeric import *
        import LinearAlgebra
        InvertMatrix = LinearAlgebra.inverse
        MatrixMulitply = matrixmultiply
        FloatType = Float
except:
    print "\n* Warning: Unable to import numpy.  LDA training not available."
    print "  Please install NumPy (see http://numpy.scipy.org/ for details)"
    print "  Error details are shown here:"
    traceback.print_exc()
    
import math
import os
import sys
import random
import struct
import traceback

ForbiddenFeatures = [2, 3, 4, 5, 13, 31, 32, 33, 34, 43, 47] #[2,3,4]

def PrintHistogram(Histogram, HistoFile):
    Bins = Histogram.keys()
    Bins.sort()
    #Bins.reverse()
    TotalBads = 0
    TotalGoods = 0
    print "\nHistogram results:"
    for Bin in Bins:
        TotalBads += Histogram[Bin][0]
    Bads = TotalBads
    for Bin in range(Bins[0], Bins[-1]):
        if Histogram.has_key(Bin):
            Bads -= Histogram[Bin][0]
        PValue = Bads / float(TotalBads)
        print "%s\t%s\t%s\t%s\t"%(Bin, PValue, Bads, TotalBads)
        if HistoFile and Bin >=- 70 and Bin < 150:
            PValue = min(0.99, max(0.0001, PValue))
            Str = struct.pack("<f", PValue)
            #print "PValue struct:", Str
            HistoFile.write(Str)
            
class LDAClassifier:
    def __init__(self):
        pass
    def GetCovarianceArray(self, VectorList):
        Size = len(VectorList[0])
        VectorCount = float(len(VectorList))
        C = zeros((Size, Size), FloatType)
        for Vector in VectorList:
            for X in range(Size):
                for Y in range(Size):
                    C[X][Y] += Vector[X] * Vector[Y] / VectorCount
        return C
    def LoadVectors(self, FileName, CategoryBit, FeatureList):
        Size = len(FeatureList)
        self.GoodVectors = []
        self.BadVectors = []
        # Iterate over file lines, and read vetors in:
        File = open(FileName, "r")
        for FileLine in File.xreadlines():
            if FileLine[0] == "#":
                continue # comment
            Bits = FileLine.split("\t")
            try:
                Category = int(Bits[CategoryBit])
            except:
                continue
            # Turn -1 vs 1 into 0 vs 1:
            if Category < 0:
                Category = 0
            #for X in range(len(Bits)):
            #    print "%s: %s"%(X, Bits[X])
            Vector = []
            try:
                for Index in FeatureList:
                    if Index >= len(Bits) or not Bits[Index].strip():
                        Vector.append(0)
                    else:
                        Vector.append(float(Bits[Index]))
            except:
                traceback.print_exc()
                print Bits
                continue
            if Category:
                self.GoodVectors.append(Vector)
            else:
                self.BadVectors.append(Vector)
        print "First GoodVector:\n", self.GoodVectors[0]
        print "First BadVector:\n", self.BadVectors[0]
    def ScaleVectors(self):
        """
        Scale all vectors so that 90% of all values lie in the range [-1, 1]
        """
        Values = []
        MinValues = []
        MaxValues = []
        FeatureCount = len(self.GoodVectors[0])
        for X in range(FeatureCount):
            Values.append([])
        for Vector in self.GoodVectors:
            for X in range(FeatureCount):
                Values[X].append(Vector[X])
        for Vector in self.BadVectors:
            for X in range(FeatureCount):
                Values[X].append(Vector[X])
        print "Value count:", len(Values[0])
        for X in range(FeatureCount):
            Values[X].sort()
            ValueCount = len(Values[X])
            MinValues.append(Values[X][int(round(ValueCount * 0.05))])
            MaxValues.append(Values[X][int(round(ValueCount * 0.95))])
        print "Range:"
        for X in range(FeatureCount):
            print "%s: %.4f ... %.4f"%(X, MinValues[X], MaxValues[X])
        for X in range(FeatureCount):
            HalfRange = (MaxValues[X] - MinValues[X]) / 2.0
            if not HalfRange:
                continue
            for Vector in self.BadVectors:            
                Vector[X] = (Vector[X] - MinValues[X]) / HalfRange - 1.0
                #Vector[X] = max(-1.0, min(Vector[X], 1.0))
            for Vector in self.GoodVectors:            
                Vector[X] = (Vector[X] - MinValues[X]) / HalfRange - 1.0
                #Vector[X] = max(-1.0, min(Vector[X], 1.0))
    def PerformLDA(self, FileName, CategoryBit, FeatureList, ScaleVectors = 1, FoldValidation = 0):
        VerboseFlag = 1
        Size = len(FeatureList)
        self.LoadVectors(FileName, CategoryBit, FeatureList)
        if ScaleVectors:
            self.ScaleVectors()
        if FoldValidation:
            random.seed(1)
            random.shuffle(self.GoodVectors)
            random.shuffle(self.BadVectors)
        # n-fold validation:
        self.MasterGoodVectors = self.GoodVectors
        self.MasterBadVectors = self.BadVectors
        WorstAccuracy = 1.0
        for Fold in range(max(1, FoldValidation)):
            # Slice the master lists of good and bad vectors to separate 1/FoldValidation of
            # them into a test set.  FoldValidation can be 0, to do no such splitting.
            self.GoodVectors = []
            self.GoodTestVectors = []
            for X in range(len(self.MasterGoodVectors)):
                if FoldValidation and X % FoldValidation == Fold:
                    self.GoodTestVectors.append(self.MasterGoodVectors[X])
                else:
                    self.GoodVectors.append(self.MasterGoodVectors[X])
            self.BadVectors = []
            self.BadTestVectors = []
            for X in range(len(self.MasterBadVectors)):
                if FoldValidation and X % FoldValidation == Fold:
                    self.BadTestVectors.append(self.MasterBadVectors[X])
                else:
                    self.BadVectors.append(self.MasterBadVectors[X])
            ############################################################
            # Compute the mean vectors:
            GoodCount = float(len(self.GoodVectors))
            BadCount = float(len(self.BadVectors))
            AllCount = GoodCount + BadCount
            self.MeanGood = [0]*Size
            self.MeanBad = [0]*Size
            self.MeanGlobal = [0]*Size
            for Vector in self.GoodVectors:
                for Index in range(Size):
                    self.MeanGood[Index] += Vector[Index] / GoodCount
                    self.MeanGlobal[Index] += Vector[Index] / AllCount
            for Vector in self.BadVectors:
                for Index in range(Size):
                    self.MeanBad[Index] += Vector[Index] / BadCount
                    self.MeanGlobal[Index] += Vector[Index] / AllCount
            print "MeanGood:\n  ", self.MeanGood
            print "MeanBad:\n  ", self.MeanBad
            print "MeanGlobal:\n  ", self.MeanGlobal
            ############################################################
            # Compute the mean-corrected vectors:
            MeanCorrectedGoodVectors = []
            MeanCorrectedBadVectors = []
            for Vector in self.GoodVectors:
                NewVector = []
                for X in range(Size):
                    NewVector.append(Vector[X] - self.MeanGlobal[X])
                MeanCorrectedGoodVectors.append(NewVector)
            for Vector in self.BadVectors:
                NewVector = []
                for X in range(Size):
                    NewVector.append(Vector[X] - self.MeanGlobal[X])
                MeanCorrectedBadVectors.append(NewVector)
            ############################################################
            # Compute covariance matrices:
            CovarArrayGood = self.GetCovarianceArray(MeanCorrectedGoodVectors)
            if VerboseFlag:
                print "CovarArrayGood:", CovarArrayGood
            CovarArrayBad = self.GetCovarianceArray(MeanCorrectedBadVectors)
            if VerboseFlag:
                print "CovarArrayBad:", CovarArrayBad
            # CovarArrayFull is the pooled within-group covariance matrix, it's
            # computed componentwise as weighted sum of CovarArrayGood and CovarArrayBad.
            CovarArrayFull = zeros((Size, Size), FloatType)
            for X in range(Size):
                for Y in range(Size):
                    CovarArrayFull[X][Y] += CovarArrayGood[X][Y] * GoodCount / AllCount
                    CovarArrayFull[X][Y] += CovarArrayBad[X][Y] * BadCount / AllCount
            if VerboseFlag:
                print "CovarArrayFull:", CovarArrayFull
            ############################################################
            # Invert the covariance array:
            try:
                self.CI = InvertMatrix(CovarArrayFull)
            except:
                traceback.print_exc()
                print "Unable to invert covariance matrix!  Invalid feature set."
                return 0
            if VerboseFlag:
                print "CI:", self.CI
            self.GoodMuC = MatrixMulitply(self.CI, self.MeanGood)
            if VerboseFlag:
                print "GoodMuC:", self.GoodMuC
            self.BadMuC = MatrixMulitply(self.CI, self.MeanBad)
            if VerboseFlag:
                print "BadMuC:", self.BadMuC
            self.ConstantGood = -MatrixMulitply(self.MeanGood, self.GoodMuC) / 2.0
            self.ConstantBad = -MatrixMulitply(self.MeanBad, self.BadMuC) / 2.0
            if VerboseFlag:
                print "Constant good %.4f constant bad %.4f"%(self.ConstantGood, self.ConstantBad)
            #######################################################
            if VerboseFlag:
                # Print C initializers:
                for X in range(Size):
                    Str = "double CovInv%s[] = {"%chr(ord("A") + X)
                    for Y in range(Size):
                        Str += "%.3f,"%self.CI[X][Y]
                    Str = Str[:-1] + "};"
                    print Str
                Str = "double MeanVectorTrue[] = {"
                for X in range(Size):
                    Str += "%.3f,"%self.MeanGood[X]
                Str = Str[:-1] + "};"
                print Str
                Str = "double MeanVectorFalse[] = {"
                for X in range(Size):
                    Str += "%.3f,"%self.MeanBad[X]
                Str = Str[:-1] + "};"
                print Str
                print "double SubProdTrue = (float)%.3f;"%self.ConstantGood
                print "double SubProdFalse = (float)%.3f;"%self.ConstantBad
                print "CG and CB:", self.ConstantGood, self.ConstantBad
            #######################################################
            Weights = []
            for X in range(Size):
                Weights.append(self.GoodMuC[X] - self.BadMuC[X])
            Str = "*-*->Weights:"
            for X in range(Size):
                Str += " %s: %.4f"%(X, Weights[X])
            print Str
            #######################################################
            # Compute our accuracy on the testing set:
            if FoldValidation:
                CorrectCount = 0
                IncorrectCount = 0
                for Vector in self.GoodTestVectors:
                    NewVector = []
                    for X in range(Size):
                        NewVector.append(Vector[X] - self.MeanGlobal[X])
                    Reading = self.GetReading(NewVector)
                    if Reading > 0:
                        CorrectCount += 1
                    else:
                        IncorrectCount += 1
                for Vector in self.BadTestVectors:
                    NewVector = []
                    for X in range(Size):
                        NewVector.append(Vector[X] - self.MeanGlobal[X])
                    Reading = self.GetReading(NewVector)
                    if Reading > 0:
                        IncorrectCount += 1
                    else:
                        CorrectCount += 1
                TotalCount = CorrectCount + IncorrectCount
                Accuracy = CorrectCount / float(TotalCount)
                print "Cross-validation accuracy: %d of %d (%.3f%%)"%(CorrectCount, TotalCount, Accuracy*100)
                WorstAccuracy = min(Accuracy, WorstAccuracy)
            else:
                # Compute accuracy on all vectors:
                CorrectCount = 0
                IncorrectCount = 0
                for Vector in MeanCorrectedGoodVectors:
                    Reading = self.GetReading(NewVector)
                    if Reading > 0:
                        CorrectCount += 1
                    else:
                        IncorrectCount += 1
                for Vector in MeanCorrectedBadVectors:
                    Reading = self.GetReading(NewVector)
                    if Reading > 0:
                        IncorrectCount += 1
                    else:
                        CorrectCount += 1
                TotalCount = CorrectCount + IncorrectCount
                Accuracy = CorrectCount / float(TotalCount)
                print "Accuracy: %d of %d (%.3f%%)"%(CorrectCount, TotalCount, Accuracy*100)
                WorstAccuracy = min(Accuracy, WorstAccuracy)
        print "Min. cross-validation accuracy: %.3f%%"%(WorstAccuracy*100)
        return WorstAccuracy
    def GetReading(self, Vector):
        CIProduct = MatrixMulitply(self.CI, Vector)
        ReadingGood = MatrixMulitply(self.MeanGood, CIProduct) + self.ConstantGood
        ReadingBad = MatrixMulitply(self.MeanBad, CIProduct) + self.ConstantBad
        print
        print "Vector:", Vector
        print "CIProduct:", CIProduct
        print "ReadingGood %s ReadingBad %s Net %s"%(ReadingGood, ReadingBad, ReadingGood - ReadingBad)
        return (ReadingGood - ReadingBad)
    def ReportROCCurve(self):
        SortedList = []
        MeanCorrectedGoodVectors = []
        MeanCorrectedBadVectors = []
        PositiveCount = len(self.GoodVectors)
        NegativeCount = len(self.BadVectors)
        Size = len(self.GoodVectors[0])
        for Vector in self.GoodVectors:
            NewVector = []
            for X in range(Size):
                NewVector.append(Vector[X] - self.MeanGlobal[X])
            CIProduct = MatrixMulitply(self.CI, NewVector)
            ReadingGood = MatrixMulitply(self.MeanGood, CIProduct) + self.ConstantGood
            ReadingBad = MatrixMulitply(self.MeanBad, CIProduct) + self.ConstantBad
            SortedList.append((ReadingGood - ReadingBad, 1))
        for Vector in self.BadVectors:
            NewVector = []
            for X in range(Size):
                NewVector.append(Vector[X] - self.MeanGlobal[X])
            CIProduct = MatrixMulitply(self.CI, NewVector)
            ReadingGood = MatrixMulitply(self.MeanGood, CIProduct) + self.ConstantGood
            ReadingBad = MatrixMulitply(self.MeanBad, CIProduct) + self.ConstantBad
            SortedList.append((ReadingGood - ReadingBad, 0))
        SortedList.sort()
        SortedList.reverse()
        TPCount = 0
        FPCount = 0
        Area = 0
        ROCCurveFile = open("ROCCurve.txt", "wb")
        for (Reading, TrueFlag) in SortedList:
            #print Reading, TrueFlag
            if (TrueFlag):
                TPCount += 1
            else:
                FPCount += 1
                Area += (TPCount / float(PositiveCount))
            TPRate = TPCount / float(PositiveCount)
            FPRate = FPCount / float(NegativeCount)
            ROCCurveFile.write("%s\t%s\t\n"%(FPRate, TPRate))
            if FPRate < 0.05:
                HappyTPRate = TPRate
        Area /= float(FPCount)
        print "ROC curve area:", Area
        print "TP rate for FP < 0.05: %s"%HappyTPRate
        return Area
    def ProducePValueCurve(self):
        Histogram = {}
        HistogramShort = {}
        HistogramMedium = {}
        HistogramLong = {}
        MediumCutoff = 9
        LongCutoff = 13
        for X in range(len(self.BadVectors)):
            Vector = self.BadVectors[X]
            MQScore = self.GetReading(Vector)
            Bin = int(round(MQScore * 10))
            if not Histogram.has_key(Bin):
                Histogram[Bin] = [0,0]
            if not HistogramShort.has_key(Bin):
                HistogramShort[Bin] = [0,0]
            if not HistogramMedium.has_key(Bin):
                HistogramMedium[Bin] = [0,0]
            if not HistogramLong.has_key(Bin):
                HistogramLong[Bin] = [0,0]
            #Len = self.BadVectorPepLengths[X]
            Histogram[Bin][0] += 1
        PrintHistogram(Histogram, None)

def FeatureSelectMain():
    BestAccuracy = 0
    FeatureList = [6, 49]
    FeatureCount = 55
    while len(FeatureList)<12:
        BestAccuracy = 0
        BestList = FeatureList
        for FeatureA in range(4, FeatureCount):
            if FeatureA in FeatureList:
                continue
            if FeatureA in ForbiddenFeatures:
                continue
            AugmentedFeatureList = FeatureList[:]
            AugmentedFeatureList.append(FeatureA)
            LDA = LDAClassifier()
            LDA.PerformLDA("TrainingSet.Table.txt", 0, AugmentedFeatureList, 1, 0)
            Accuracy = LDA.ReportROCCurve()
            if Accuracy > BestAccuracy:
                BestAccuracy = Accuracy
                BestList = AugmentedFeatureList
            print "Feature set %s has accuracy %.4f%%"%(AugmentedFeatureList, Accuracy*100)
            print "So far...best accuracy %.4f%%, feature set %s"%(BestAccuracy*100, BestList)
        FeatureList = BestList
        print "Best accuracy %s, feature list %s"%(BestAccuracy, FeatureList)
            

def Main():
    LDA = LDAClassifier()
    LDA.PerformLDA("LDATrainingSet.txt", 0, [1, 5], 0, 0)
    LDA.ReportROCCurve()
    #LDA.ProducePValueCurve()
        
if __name__ == "__main__":
    try:
        import psyco
        psyco.full()
    except:
        print "(psyco not loaded)"
    Main()



#Title:          ReleasePySVM.py
#Author:         Stephen Tanner, Samuel Payne, Natalie Castellana, Pavel Pevzner, Vineet Bafna
#Created:        2005
# Copyright 2007,2008,2009 The Regents of the University of California
# All Rights Reserved
#
# Permission to use, copy, modify and distribute any part of this
# program for educational, research and non-profit purposes, by non-profit
# institutions only, without fee, and without a written agreement is hereby
# granted, provided that the above copyright notice, this paragraph and
# the following three paragraphs appear in all copies.
#
# Those desiring to incorporate this work into commercial
# products or use for commercial purposes should contact the Technology
# Transfer & Intellectual Property Services, University of California,
# San Diego, 9500 Gilman Drive, Mail Code 0910, La Jolla, CA 92093-0910,
# Ph: (858) 534-5815, FAX: (858) 534-7345, E-MAIL:invent@ucsd.edu.
#
# IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
# FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
# INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
# OF SUCH DAMAGE.
#
# THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
# OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
# ENHANCEMENTS, OR MODIFICATIONS.  THE UNIVERSITY OF CALIFORNIA MAKES NO
# REPRESENTATIONS AND EXTENDS NO WARRANTIES OF ANY KIND, EITHER IMPLIED OR
# EXPRESS, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, OR THAT THE USE OF
# THE SOFTWARE WILL NOT INFRINGE ANY PATENT, TRADEMARK OR OTHER RIGHTS.

"""
Script to build PySVM
"""
import sys
import distutils.core

def Main(Arguments):
    print "Prepping PySVM..."
    PySVMFileNames = ['PySVM/PySVM.c',"PySVM/svm-predict.c", "PySVM/svm.cpp",]
    PySVMExtension = distutils.core.Extension('PySVM', sources = PySVMFileNames)
    distutils.core.setup(name = 'PySVM', version = '1.0', ext_modules = [PySVMExtension],
          script_args = Arguments)

if __name__ == "__main__":
    Main(sys.argv[1:])
    



#Title:          ParseXML.py
#Author:         Stephen Tanner, Samuel Payne, Natalie Castellana, Pavel Pevzner, Vineet Bafna
#Created:        2005
# Copyright 2007,2008,2009 The Regents of the University of California
# All Rights Reserved
#
# Permission to use, copy, modify and distribute any part of this
# program for educational, research and non-profit purposes, by non-profit
# institutions only, without fee, and without a written agreement is hereby
# granted, provided that the above copyright notice, this paragraph and
# the following three paragraphs appear in all copies.
#
# Those desiring to incorporate this work into commercial
# products or use for commercial purposes should contact the Technology
# Transfer & Intellectual Property Services, University of California,
# San Diego, 9500 Gilman Drive, Mail Code 0910, La Jolla, CA 92093-0910,
# Ph: (858) 534-5815, FAX: (858) 534-7345, E-MAIL:invent@ucsd.edu.
#
# IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
# FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
# INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
# OF SUCH DAMAGE.
#
# THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
# OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
# ENHANCEMENTS, OR MODIFICATIONS.  THE UNIVERSITY OF CALIFORNIA MAKES NO
# REPRESENTATIONS AND EXTENDS NO WARRANTIES OF ANY KIND, EITHER IMPLIED OR
# EXPRESS, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, OR THAT THE USE OF
# THE SOFTWARE WILL NOT INFRINGE ANY PATENT, TRADEMARK OR OTHER RIGHTS.


"""
MZXML and mzData peak parsing
"""

import os
import sys
import struct
import xml.sax
import xml.sax.handler
import base64
import MSSpectrum

if hasattr(base64, "b64decode"):
    B64Decode = base64.b64decode
    B64Encode = base64.b64encode
else:
    B64Decode = base64.decodestring
    B64Encode = base64.encodestring

def GetSpectrumPeaksMZXML(Spectrum, File):
    Spectrum.Peaks = []
    SAXParser = xml.sax.make_parser()
    Handler = MZXMLPeakParser(Spectrum)
    Handler.Parser = SAXParser
    SAXParser.setContentHandler(Handler)
    try:
        SAXParser.parse(File)
    except xml.sax.SAXException, XMLException:
        Message = XMLException.getMessage()
        # If there are no peaks, then all exceptions are raised:
        if not len(Spectrum.Peaks):
            raise
        # If we did succeed in getting peaks, then the error likely arose
        # after the end of the peaks tag.
        if Message == "junk after document element":
            pass
        elif Message == "not well-formed (invalid token)":
            pass
        else:
            raise

def GetSpectrumPeaksMZData(Spectrum, File):
    Spectrum.Peaks = []
    SAXParser = xml.sax.make_parser()
    Handler = MZDataPeakParser(Spectrum)
    Handler.Parser = SAXParser
    SAXParser.setContentHandler(Handler)
    try:
        SAXParser.parse(File)
    except xml.sax.SAXException, XMLException:
        Message = XMLException.getMessage()
        if Message == "junk after document element":
            pass
        elif Message == "not well-formed (invalid token)":
            pass
        else:
            raise

class MZXMLParseStates:
    SpectrumComplete = -1
    Skipping = 0
    Peaks = 1
    PrecursorMZ = 2

class XMLDictionaryHandler(xml.sax.handler.ContentHandler):
    """
    A simple wrapper for the skeletal ContentHandler class.  Fixes broken API names, and
    supports the use of "triage dictionaries" self.StartHandlers and self.EndHandlers
    to find the handlers for tags.
    """
    def __init__(self):
        # Repair names:
        self.startElement = self.StartElement
        self.endElement = self.EndElement
        self.characters = self.HandleCharacters
        #
        self.VerboseFlag = 0
    def StartElement(self, Name, Attributes):
        #print "Start <%s>@%s"%(Name, self.Parser._parser.CurrentByteIndex)
        Handler = self.StartHandlers.get(Name, None)
        if self.VerboseFlag:
            print "<%s> %s"%(Name, Handler)
        if Handler:
            Handler(Attributes)
    def EndElement(self, Name):
        #print "  End <%s>"%Name
        Handler = self.EndHandlers.get(Name, None)
        if self.VerboseFlag:
            print "</%s> %s"%(Name, Handler)
        if Handler:
            Handler()
    def HandleCharacters(self, String):
        pass
    
class MZXMLPeakParser(XMLDictionaryHandler):
    def __init__(self, Spectrum):
        self.State = MZXMLParseStates.Skipping
        self.StartHandlers = {"peaks":self.StartPeaks,
                              "precursorMz":self.StartPrecursorMZ
                              }
        self.EndHandlers = {"peaks":self.EndPeaks,
                            "scan":self.EndScan,
                            "precursorMz":self.EndPrecursorMZ
                            }
        self.Spectrum = Spectrum
        self.PeakBuffer = ""
        XMLDictionaryHandler.__init__(self)
    def HandleCharacters(self, String):
        if self.State == MZXMLParseStates.PrecursorMZ:
            self.PrecursorMZBuffer += String
            return
        if self.State == MZXMLParseStates.Peaks:
            self.PeakBuffer += String
            return
    def StartPrecursorMZ(self, Attributes):
        if self.State == MZXMLParseStates.SpectrumComplete:
            return
        self.State = MZXMLParseStates.PrecursorMZ
        self.PrecursorMZBuffer = ""
    def EndPrecursorMZ(self):
        if self.State == MZXMLParseStates.SpectrumComplete:
            return
        #print "Precursor MZ -> %s"%self.PrecursorMZBuffer
        self.Spectrum.PrecursorMZ = float(self.PrecursorMZBuffer)
        self.State = MZXMLParseStates.Skipping
    def StartPeaks(self, Attributes):
        if self.State == MZXMLParseStates.SpectrumComplete:
            return
        self.State = MZXMLParseStates.Peaks
        self.PeakBuffer = ""
        ByteOrder = Attributes.get("byteOrder", "network")
        if ByteOrder == "little" or ByteOrder == "little-endian":
            self.ByteOrder = "little"
        else:
            self.ByteOrder = "big"
    def EndScan(self):
        self.State = MZXMLParseStates.SpectrumComplete
    def EndPeaks(self):
        if self.State == MZXMLParseStates.SpectrumComplete:
            return
        DecodedPeaks = B64Decode(self.PeakBuffer)
        StringPos = 0
        self.Peaks = []
        while StringPos < len(DecodedPeaks):
            if self.ByteOrder == sys.byteorder:
                Mass = struct.unpack("f", DecodedPeaks[StringPos:StringPos+4])[0]
                Intensity = struct.unpack("f", DecodedPeaks[StringPos+4:StringPos+8])[0]                
            else:
                Mass = struct.unpack("!f", DecodedPeaks[StringPos:StringPos+4])[0]
                Intensity = struct.unpack("!f", DecodedPeaks[StringPos+4:StringPos+8])[0]
            Peak = MSSpectrum.PeakClass(Mass, Intensity)
            StringPos += 8
            #print Peak.Mass, Peak.Intensity
            self.Spectrum.Peaks.append(Peak)

class MZDataParseStates:
    SpectrumComplete = -1
    Skipping = 0
    MZArray = 1
    MZArrayData = 2
    IntensityArray = 3
    IntensityArrayData = 4

class MZDataPeakParser(XMLDictionaryHandler):
    def __init__(self, Spectrum):
        self.State = MZDataParseStates.Skipping
        self.StartHandlers = {"data":self.StartData,
                              "mzArrayBinary":self.StartMZArrayBinary,
                              "intenArrayBinary":self.StartIntensityArrayBinary,
                              "cvParam":self.StartCVParam,
                              }
        self.EndHandlers = {"data":self.EndData,
                            "spectrum":self.EndSpectrum,
                            }
        self.Spectrum = Spectrum
        self.PeakBuffer = ""
        XMLDictionaryHandler.__init__(self)
    def StartCVParam(self, Attributes):
        Name = Attributes.get("name", None)
        Value = Attributes.get("value", None)
        if Name == "mz":
            self.Spectrum.PrecursorMZ = float(Value)
    def StartMZArrayBinary(self, Attributes):
        if self.State == MZDataParseStates.SpectrumComplete:
            return
        self.State = MZDataParseStates.MZArray
    def StartIntensityArrayBinary(self, Attributes):
        if self.State == MZDataParseStates.SpectrumComplete:
            return
        self.State = MZDataParseStates.IntensityArray
    def HandleCharacters(self, String):
        if self.State in (MZDataParseStates.MZArrayData, MZDataParseStates.IntensityArrayData):
            self.PeakBuffer += String
    def EndData(self):
        if self.State in (MZDataParseStates.MZArrayData, MZDataParseStates.IntensityArrayData):
            # Parse the float array:
            FloatList = []
            DecodedPeaks = B64Decode(self.PeakBuffer)
            StringPos = 0
            while StringPos < len(DecodedPeaks):
                if self.ByteOrder == sys.byteorder:
                    if self.Precision == 64:
                        Value = struct.unpack("d", DecodedPeaks[StringPos:StringPos + 8])[0]
                    else:
                        Value = struct.unpack("f", DecodedPeaks[StringPos:StringPos + 4])[0]
                else:
                    if self.Precision == 64:
                        Value  = struct.unpack("!d", DecodedPeaks[StringPos:StringPos + 8])[0]
                    else:
                        Value = struct.unpack("!f", DecodedPeaks[StringPos:StringPos + 4])[0]
                #Peak = MSSpectrum.PeakClass(Mass, Intensity)
                FloatList.append(Value)
                if self.Precision == 64:
                    StringPos += 8
                else:
                    StringPos += 4
                #print Peak.Mass, Peak.Intensity
                #self.Spectrum.Peaks.append(Peak)
            if self.State == MZDataParseStates.MZArrayData:
                self.MZList = FloatList
            else:
                self.IntensityList = FloatList
            #print "...parsed %s values!"%len(FloatList)
            self.State = MZDataParseStates.Skipping
    def StartData(self, Attributes):
        if self.State == MZDataParseStates.SpectrumComplete:
            return
        self.Precision = Attributes.get("precision", "32")
        ByteOrder = Attributes.get("endian", "network")
        if ByteOrder == "little" or ByteOrder == "little-endian":
            self.ByteOrder = "little"
        else:
            self.ByteOrder = "big"
        if self.State == MZDataParseStates.MZArray:
            self.State = MZDataParseStates.MZArrayData
            self.PeakBuffer = ""
            return
        if self.State == MZDataParseStates.IntensityArray:
            self.State = MZDataParseStates.IntensityArrayData
            self.PeakBuffer = ""
    def EndSpectrum(self):
        if self.State != MZXMLParseStates.SpectrumComplete:
            self.State = MZXMLParseStates.SpectrumComplete
            for PeakIndex in range(len(self.MZList)):
                Mass = self.MZList[PeakIndex]
                Intensity = self.IntensityList[PeakIndex]
                Peak = MSSpectrum.PeakClass(Mass, Intensity)
                self.Spectrum.Peaks.append(Peak)



#Title:          BuildInspect.py
#Author:         Stephen Tanner, Samuel Payne, Natalie Castellana, Pavel Pevzner, Vineet Bafna
#Created:        2005
# Copyright 2007,2008,2009 The Regents of the University of California
# All Rights Reserved
#
# Permission to use, copy, modify and distribute any part of this
# program for educational, research and non-profit purposes, by non-profit
# institutions only, without fee, and without a written agreement is hereby
# granted, provided that the above copyright notice, this paragraph and
# the following three paragraphs appear in all copies.
#
# Those desiring to incorporate this work into commercial
# products or use for commercial purposes should contact the Technology
# Transfer & Intellectual Property Services, University of California,
# San Diego, 9500 Gilman Drive, Mail Code 0910, La Jolla, CA 92093-0910,
# Ph: (858) 534-5815, FAX: (858) 534-7345, E-MAIL:invent@ucsd.edu.
#
# IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
# FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
# INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
# OF SUCH DAMAGE.
#
# THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
# OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
# ENHANCEMENTS, OR MODIFICATIONS.  THE UNIVERSITY OF CALIFORNIA MAKES NO
# REPRESENTATIONS AND EXTENDS NO WARRANTIES OF ANY KIND, EITHER IMPLIED OR
# EXPRESS, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, OR THAT THE USE OF
# THE SOFTWARE WILL NOT INFRINGE ANY PATENT, TRADEMARK OR OTHER RIGHTS.

"""
Python script to build Inspect.  An alternative to makefiles.
"""
import sys
import distutils
import distutils.command.build
import distutils.ccompiler

def BuildInspect(BuildNow = 0):
    InspectSourceFiles = [
        "base64.c", "BN.c", "BuildMS2DB.c", "ChargeState.c", "CMemLeak.c",
        "Errors.c", "ExonGraphAlign.c", 
        "FreeMod.c", "IonScoring.c", "LDA.c", "main.c", "Mods.c",
        "MS2DB.c", "ParentMass.c", "ParseInput.c", 
        "ParseXML.c", "PValue.c", 
        "Run.c", "Score.c", "Scorpion.c", "SNP.c", "Spectrum.c", "Spliced.c", 
        "SpliceDB.c", "SpliceScan.c", "SVM.c", "Tagger.c", "Trie.c", "Utils.c", "TagFile.c"
        ]
    ExtraIncludeDirectories = ["expat\\lib",]
    class MyBuildClass(distutils.command.build.build):
        def build_opt(self):
            CC = distutils.ccompiler.new_compiler()
            #if sys.platform != 'win32':
            #    CC.add_library('m')
            #import os.path
            print dir(CC)
            CC.library_dirs.append("expat/lib/release")
            if sys.platform == "win32":
                CC.add_library("libexpat")
            else:
                CC.add_library("expat") # not "libexpat", that won't work on Linux.
                CC.add_library("m")
            CC.set_include_dirs(ExtraIncludeDirectories)
            opt_obj = CC.compile(InspectSourceFiles)
            CC.link_executable(opt_obj, "inspect")
        def run(self):
            self.build_opt()
            distutils.command.build.build.run(self)
    if BuildNow:
        Dist = distutils.dist.Distribution()
        Dist.parse_config_files()
        Dist.cmdclass["build"] = MyBuildClass
        Dist.commands = ["build"]
        Dist.run_commands()
    else:
        distutils.core.setup(cmdclass = {"build":MyBuildClass,})

def BuildInspectOnConvey(BuildNow = 0):
    InspectSourceFiles = [
        "base64.c", "BN.c", "BuildMS2DB.c", "ChargeState.c", "CMemLeak.c",
        "Errors.c", "ExonGraphAlign.c", 
        "FreeMod.c", "IonScoring.c", "LDA.c", "main.c", "Mods.c",
        "MS2DB.c", "ParentMass.c", "ParseInput.c", 
        "ParseXML.c", "PValue.c", 
        "Run.c", "Score.c", "Scorpion.c", "SNP.c", "Spectrum.c", "Spliced.c", 
        "SpliceDB.c", "SpliceScan.c", "SVM.c", "Tagger.c", "Trie.c", "Utils.c", "TagFile.c",
        "cny_kernel_wrapper.c", "pdk_kernel.c", "kernel.c", "cny_util.c"]
    ExtraIncludeDirectories = ["expat\\lib",]
    class MyBuildClass(distutils.command.build.build):
        def build_opt(self):
            CC = distutils.ccompiler.new_compiler()
            #if sys.platform != 'win32':
            #    CC.add_library('m')
            #import os.path
            print dir(CC)
            CC.library_dirs.append("expat/lib/release")
            if sys.platform == "win32":
                CC.add_library("libexpat")
            else:
                CC.add_library("expat") # not "libexpat", that won't work on Linux.
                CC.add_library("m")
            CC.set_include_dirs(ExtraIncludeDirectories)
            opt_obj = CC.compile(InspectSourceFiles)
            CC.link_executable(opt_obj, "inspect")
        def run(self):
            self.build_opt()
            distutils.command.build.build.run(self)
    if BuildNow:
        Dist = distutils.dist.Distribution()
        Dist.parse_config_files()
        Dist.cmdclass["build"] = MyBuildClass
        Dist.commands = ["build"]
        Dist.run_commands()
    else:
        distutils.core.setup(cmdclass = {"build":MyBuildClass,})
    

if __name__ == "__main__":
    #sys.argv = ["", "build"]
    BuildInspect()
    



#Title:          PValue.py
#Author:         Stephen Tanner, Samuel Payne, Natalie Castellana, Pavel Pevzner, Vineet Bafna
#Created:        2010
# Copyright 2007,2008,2009 The Regents of the University of California
# All Rights Reserved
#
# Permission to use, copy, modify and distribute any part of this
# program for educational, research and non-profit purposes, by non-profit
# institutions only, without fee, and without a written agreement is hereby
# granted, provided that the above copyright notice, this paragraph and
# the following three paragraphs appear in all copies.
#
# Those desiring to incorporate this work into commercial
# products or use for commercial purposes should contact the Technology
# Transfer & Intellectual Property Services, University of California,
# San Diego, 9500 Gilman Drive, Mail Code 0910, La Jolla, CA 92093-0910,
# Ph: (858) 534-5815, FAX: (858) 534-7345, E-MAIL:invent@ucsd.edu.
#
# IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
# FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
# INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
# OF SUCH DAMAGE.
#
# THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
# OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
# ENHANCEMENTS, OR MODIFICATIONS.  THE UNIVERSITY OF CALIFORNIA MAKES NO
# REPRESENTATIONS AND EXTENDS NO WARRANTIES OF ANY KIND, EITHER IMPLIED OR
# EXPRESS, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, OR THAT THE USE OF
# THE SOFTWARE WILL NOT INFRINGE ANY PATENT, TRADEMARK OR OTHER RIGHTS.


"""
This script, based on PeptideProphet, computes an F-Score.  The F-Score
for a match is a weighted sum of the length-corrected MQScore and the delta score.

Also, there is no length constraint on peptides.  An FScore is computed for all peptides.
The FScore column of each peptide-spectrum match (PSM) is updated, but no p-value or FDR is caluclated.

"""
import os
import sys
import random
import math
import getopt
import traceback
import struct
import ResultsParser
import SelectProteins
import Learning
from Utils import *
Initialize()

class Defaults:
    MQScoreWeight = 0.3
    DeltaScoreWeight = 1.5
    ###########################
    BlindMQScoreWeight = 0.3
    BlindDeltaScoreWeight = 1.5
    
    ###########################
    

BLIND_MOD_PENALTY = 1.0
MIN_MQSCORE = -10.0

# Parse the scores from at most this many output files.  
MAX_RESULTS_FILES_TO_PARSE = 100

BIN_MULTIPLIER = 10.0
SQRT2PI = math.sqrt(2 * math.pi)

Cof = [76.18009172947146, -86.50532032941677,
    24.01409824083091, -1.231739572450155, 
    0.1208650973866179e-2, -0.5395239384952e-5]

class Bag:
    pass

class FScoreParser(ResultsParser.ResultsParser):
    def __init__(self):
        
        self.VerboseFlag = 0
        self.MQScoreWeight = Defaults.MQScoreWeight
        self.DeltaScoreWeight = Defaults.DeltaScoreWeight
        self.BlindFlag = 0
        self.MaxDeltaScoreGap = -3.5
        self.SplitByCharge = 0
        self.Columns = ResultsParser.Columns()
        ResultsParser.ResultsParser.__init__(self)

    def ReadDeltaScoreDistribution(self, FilePath):
        """
        Read delta-scores from a file, to compute the average delta-score.
        If passed a directory, iterate over all results files in the directory.
        """
        #
        self.AllSpectrumCount2 = 0
        self.AllSpectrumCount3 = 0
        self.MeanDeltaScore2 = 0
        self.MeanDeltaScore3 = 0
        self.ProcessResultsFiles(FilePath, self.ReadDeltaScoreDistributionFromFile, MAX_RESULTS_FILES_TO_PARSE)
        
        if self.SplitByCharge == 1:
            self.MeanDeltaScore2 /= max(1, self.AllSpectrumCount2)
            self.MeanDeltaScore3 /= max(1, self.AllSpectrumCount3)
            if self.VerboseFlag:
                print "Mean delta score ch1..2: %s over %s spectra"%(self.MeanDeltaScore2, self.AllSpectrumCount2)
                print "Mean delta score ch3: %s over %s spectra"%(self.MeanDeltaScore3, self.AllSpectrumCount3)
            if not self.MeanDeltaScore2:
                self.MeanDeltaScore2 = 0.001
            if not self.MeanDeltaScore3:
                self.MeanDeltaScore3 = 0.001
        else:
            self.MeanDeltaScore = (self.MeanDeltaScore2 + self.MeanDeltaScore3)/(max(1,self.AllSpectrumCount2+self.AllSpectrumCount3))
            if self.VerboseFlag:
                print "Mean delta score: %s over %s spectra"%(self.MeanDeltaScore, self.AllSpectrumCount2+self.AllSpectrumCount3)

    def ReadDeltaScoreDistributionFromFile(self, FilePath):
        "Read delta-scores from a single file, to compute the average delta-score."
        print "Read delta-score distribution from %s..."%FilePath
        try:
            File = open(FilePath, "rb")
        except:
            traceback.print_exc()
            return
        OldSpectrum = None
        for FileLine in File.xreadlines():
            # Skip header lines and blank lines
            if FileLine[0] == "#":
                self.Columns.initializeHeaders(FileLine)
                continue
            if not FileLine.strip():
                continue
            Bits = list(FileLine.split("\t"))
            if len(Bits) <= self.Columns.getIndex("DeltaScore"):
                continue
            try:
                Charge = int(Bits[self.Columns.getIndex("Charge")])
                MQScore = float(Bits[self.Columns.getIndex("MQScore")])
                DeltaScore = float(Bits[self.Columns.getIndex("DeltaScore")])
                Peptide = GetPeptideFromModdedName(Bits[self.Columns.getIndex("Annotation")])
                Spectrum = (os.path.basename(Bits[self.Columns.getIndex("SpectrumFile")]), Bits[self.Columns.getIndex("Scan#")])
            except:
                traceback.print_exc()
                print Bits
                continue # header line
            if Spectrum == OldSpectrum:
                continue
            
            OldSpectrum = Spectrum
            
            if DeltaScore < 0:
                print "## Warning: DeltaScore < 0!", Spectrum, FilePath
                print DeltaScore
                print MQScore
                print Bits
                raw_input()
                continue
            if Charge < 3:
                self.AllSpectrumCount2 += 1
                self.MeanDeltaScore2 += DeltaScore
                
            else:
                self.AllSpectrumCount3 += 1
                self.MeanDeltaScore3 += DeltaScore
        File.close()            
    def WriteMatchesForSpectrum(self, MatchesForSpectrum, OutFile):
        
        for Match in MatchesForSpectrum:
            # Skip short matches:
            Length = len(Match.Peptide.Aminos)
            
            if self.SplitByCharge:
                if Match.Charge < 3:
                    CurrMeanDeltaScore = self.MeanDeltaScore2
                else:
                    CurrMeanDeltaScore = self.MeanDeltaScore3

            else:
                CurrMeanDeltaScore = self.MeanDeltaScore

            WeightedScore = self.MQScoreWeight * Match.MQScore + self.DeltaScoreWeight * (Match.DeltaScore / CurrMeanDeltaScore)
            ScoreBin = int(round(WeightedScore * BIN_MULTIPLIER))
            
            Match.Bits[self.Columns.getIndex("F-Score")] = "%s"%WeightedScore
    
            OutFile.write(string.join(Match.Bits, "\t"))
            OutFile.write("\n")

    def WriteFixedScores(self, OutputPath):
       
        self.WriteScoresPath = OutputPath
        # Make the output directory, if it doesn't exist already.
        # Assume: OutputPath is a directory if ReadScoresPath is a directory,
        # and OutputPath is a file if ReadScoresPath is a file.
        if os.path.isdir(self.ReadScoresPath):
            DirName = OutputPath
        else:
            DirName = os.path.split(OutputPath)[0]
        try:
            os.makedirs(DirName)
        except:
            pass
        self.ProcessResultsFiles(self.ReadScoresPath, self.WriteFixedScoresFile)
        

    def WriteFixedScoresFile(self, Path):
        if os.path.isdir(self.ReadScoresPath):
            OutputPath = os.path.join(self.WriteScoresPath, os.path.split(Path)[1])
        else:
            OutputPath = self.WriteScoresPath
        
        try:
            InFile = open(Path, "rb")
            OutFile = open(OutputPath, "wb")
            LineCount = 0
            
            OldSpectrum = None
            MatchesForSpectrum = []
            for FileLine in InFile:
                # Lines starting with # are comments (e.g. header line), and are written out as-is:
                if FileLine[0] == "#":
                    self.Columns.initializeHeaders(FileLine)
                    OutFile.write(FileLine)
                    continue
                Bits = list(FileLine.strip().split("\t"))
                Match = Bag()
                try:
                    Match.Bits = Bits
                    Match.Charge = int(Bits[self.Columns.getIndex("Charge")])
                    Match.MQScore = float(Bits[self.Columns.getIndex("MQScore")])
                    Match.DeltaScore = float(Bits[self.Columns.getIndex("DeltaScore")])
                    Match.Peptide = GetPeptideFromModdedName(Bits[self.Columns.getIndex("Annotation")])
                    Match.ProteinName = Bits[self.Columns.getIndex("Protein")]
                except:
                    continue
                LineCount += 1
                Spectrum = (Bits[0], Bits[1])
                if Spectrum != OldSpectrum:
                    self.WriteMatchesForSpectrum(MatchesForSpectrum, OutFile)
                    MatchesForSpectrum = []
                OldSpectrum = Spectrum
                MatchesForSpectrum.append(Match)
            # Finish the last spectrum:
            self.WriteMatchesForSpectrum(MatchesForSpectrum, OutFile)
            InFile.close()
            OutFile.close()
            
            
        except:
            traceback.print_exc()
            print "* Error filtering annotations from '%s' to '%s'"%(Path, OutputPath)

    def Run(self):
        self.ReadDeltaScoreDistribution(self.ReadScoresPath)
        self.WriteFixedScores(self.WriteScoresPath)

    def ParseCommandLine(self, Arguments):
        (Options, Args) = getopt.getopt(Arguments, "r:w:cvb")
        OptionsSeen = {}

        for (Option, Value) in Options:
            OptionsSeen[Option] = 1
            if Option == "-b":
                self.BlindFlag = 1
                self.MQScoreWeight = Defaults.BlindMQScoreWeight
                self.DeltaScoreWeight = Defaults.BlindDeltaScoreWeight
                
            elif Option == "-r":
                self.ReadScoresPath  = Value
            elif Option == "-w":
                self.WriteScoresPath = Value
            elif Option == "-c":
                self.SplitByCharge = 1
            elif Option == "-v":
                self.VerboseFlag = 1
            else:
                print "** Unknown option:", Option, Value
        # Check validity of options:
        if not OptionsSeen.has_key("-r") or not OptionsSeen.has_key("-w"):
            print "* Error: Missing Arguments"
            return 0
        # No major problems - return TRUE for success.
        return 1

UsageInfo = """
ComputeFScore.py - Compute FScore based on match quality score (MQScore) and delta score.  
Write out an updated results file.

Required Parameters:
 -r [FILENAME] Read results from filename (and fit the probability mixture
    model to these results).  If the option value is a directory, we'll read
    all the results-files from the directory.
 -w [FILENAME] Write re-scored results to a file.
 
Optional Parameters:
 -c Split by charge (compute the FScore separately for charge 1 and 2, and for charge 3.
 -b Results are from a blind search (not recommended)

Internal use only:
 -v Verbose output (for debugging)

    
Example:
  ComputeFScore.py -r ShewanellaResults -w ShewanellaFiltered -c
"""

def Main(Parser = None):
    global MAX_RESULTS_FILES_TO_PARSE
    
    if not Parser:
        Parser = FScoreParser()
        Result = Parser.ParseCommandLine(sys.argv[1:])
        if not Result:
            print UsageInfo
            return
        Parser.Run()
    
if __name__ == "__main__":
    try:
        import psyco
        psyco.full()
    except:
        print "psyco not found - running without optimization"
    #TestMain()
    Main()



#Title:          SpectralSimilarity.py
#Author:         Stephen Tanner, Samuel Payne, Natalie Castellana, Pavel Pevzner, Vineet Bafna
#Created:        2005
# Copyright 2007,2008,2009 The Regents of the University of California
# All Rights Reserved
#
# Permission to use, copy, modify and distribute any part of this
# program for educational, research and non-profit purposes, by non-profit
# institutions only, without fee, and without a written agreement is hereby
# granted, provided that the above copyright notice, this paragraph and
# the following three paragraphs appear in all copies.
#
# Those desiring to incorporate this work into commercial
# products or use for commercial purposes should contact the Technology
# Transfer & Intellectual Property Services, University of California,
# San Diego, 9500 Gilman Drive, Mail Code 0910, La Jolla, CA 92093-0910,
# Ph: (858) 534-5815, FAX: (858) 534-7345, E-MAIL:invent@ucsd.edu.
#
# IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
# FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
# INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
# OF SUCH DAMAGE.
#
# THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
# OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
# ENHANCEMENTS, OR MODIFICATIONS.  THE UNIVERSITY OF CALIFORNIA MAKES NO
# REPRESENTATIONS AND EXTENDS NO WARRANTIES OF ANY KIND, EITHER IMPLIED OR
# EXPRESS, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, OR THAT THE USE OF
# THE SOFTWARE WILL NOT INFRINGE ANY PATENT, TRADEMARK OR OTHER RIGHTS.


"""
This is an auxillary module for ComputePTMFeatures.py.  It measurs the similarity between
two spectra.  The simplest case is a comparison of two spectra with the same annotation.
We are also able to compare two spectra which differ by a PTM (provided the PTM position
is known).  
"""

import MSSpectrum
import Label
import math
from Utils import *
Initialize()

class SpectralSimilarity:
    """
    Container class that holds the spectra, and measures similarity
    ASSUMPTION: Either AnnotationA and AnnotationB are the same,
    or AnnotationA is modified and AnnotationB is NOT.
    """
    def __init__(self, SpectrumA, SpectrumB, AnnotationA, AnnotationB):
        """
        SpectrumA and SpectrumB are SpectrumClass objects from MSSpectrum.py
        AnnotationA and AnnotationB are strings of the Inspect annotation of the
        spectrum (if the annotation was R.THISPEPK.M  this method would get
        passed "THISPEPK")
        """
        self.BinMultiplier = 1.0 # default
        # Spectra can be either MSSpectrum objects or file paths:
        if isinstance(SpectrumA, MSSpectrum.SpectrumClass):
            self.SpectrumA = SpectrumA
        else:
            Spectrum = MSSpectrum.SpectrumClass()
            Spectrum.ReadPeaks(SpectrumA)
            self.SpectrumA = Spectrum
            self.SpectrumA.FilterPeaks() 
            self.SpectrumA.RankPeaksByIntensity()
        if isinstance(SpectrumB, MSSpectrum.SpectrumClass):
            self.SpectrumB = SpectrumB
        else:
            Spectrum = MSSpectrum.SpectrumClass()
            Spectrum.ReadPeaks(SpectrumB)
            self.SpectrumB = Spectrum
            self.SpectrumB.FilterPeaks()
            self.SpectrumB.RankPeaksByIntensity()            
        # Annotations can be either a string, or a peptide object:
        if isinstance(AnnotationA, PeptideClass):
            self.PeptideA = AnnotationA
        else:
            self.PeptideA = GetPeptideFromModdedName(AnnotationA)
        if isinstance(AnnotationB, PeptideClass):
            self.PeptideB = AnnotationB
        else:
            self.PeptideB = GetPeptideFromModdedName(AnnotationB)
    def ComputeSimilarity(self):
        """
        This method determines how similar two spectra are.  It can use a variety of methods,
        and honestly this may turn into a big switch box
        """
        DPSimScore = self.DotProduct()
        print "The Scaled Dot Product of these two is %f"%DPSimScore
    def LabelPeaks(self, PeakTolerance = 0.5):
        """
        Should be called once, *if* the annotations differ, so that we can match
        corresponding peaks which have different masses due to PTM attachment.
        """
        # Label the spectra so that I can know which peaks belong to what name eg. b7
        Label.LabelSpectrum(self.SpectrumA, self.PeptideA, PeakTolerance)
        Label.LabelSpectrum(self.SpectrumB, self.PeptideB, PeakTolerance)
    def DotProductSignal(self, BinMultiplier = 1.0, MaxIntensityRank = 50, EnableShift = 1, VerboseFlag = 0, HashByRank = 0):
        """
        Variant of Dot Product, incorporating a correction factor introduced in [Parag and Mallick, 2006ish]
        """
        self.MaxIntensityRank = MaxIntensityRank
        self.BinMultiplier = BinMultiplier
        self.HashByRank = HashByRank
        # set up hashes
        HashA = {}
        HashB = {}
        # Populate HashA and HashB:
        if EnableShift and len(self.PeptideA.Modifications) > 0:
            self.HashPeaksWithShift(HashA, self.SpectrumA, self.PeptideA, VerboseFlag)
        else:
            self.HashPeaks(HashA, self.SpectrumA)
        if EnableShift and len(self.PeptideB.Modifications) > 0:
            self.HashPeaksWithShift(HashB, self.SpectrumB, self.PeptideB, VerboseFlag)
        else:
            self.HashPeaks(HashB, self.SpectrumB)
        #Do Dot Product
        MaxBins = max(HashA.keys())
        MaxBins = max(MaxBins, max(HashB.keys()))
        DotProduct = 0
        SumSqA = 0
        SumSqB = 0
        TotalIntensityA = 0
        TotalIntensityB = 0
        for I in range(MaxBins):
            A = HashA.get(I, 0)
            B = HashB.get(I, 0)
            TotalIntensityA += A
            TotalIntensityB += B
            if VerboseFlag and (A or B):
                print "%s\t%s\t%s\t%s\t"%(I, A, B, A*B)
            DotProduct += A * B
            SumSqA += A * A
            SumSqB += B * B
        #print "Dot Product is %f"%DotProduct
        #print "sqrt thing is %f"%math.sqrt(SumSqA * SumSqB)
        OddsCollision = 1.0 / (max(self.SpectrumA.ParentMass, self.SpectrumB.ParentMass) * BinMultiplier)
        DotProduct -= TotalIntensityA * TotalIntensityB * OddsCollision
        return DotProduct / math.sqrt(SumSqA * SumSqB)
    def DotProduct(self, BinMultiplier = 1.0, MaxIntensityRank = 50, EnableShift = 1, VerboseFlag = 0, HashByRank = 0):
        """
        This method measures similarity between spectra by calculating their dot product.
        It is written to work on spectra might be PTM shifted versions of each other.
        e.g. SAMMY and SAM+16MY.  If the peptide annotation has a PTM, then the peaks
        are shifted back.
        Variables:
        Rank = indicates that you want to use a rank based dotproduct, instead of intensity based
        """
        self.MaxIntensityRank = MaxIntensityRank
        self.BinMultiplier = BinMultiplier
        self.HashByRank = HashByRank
        # set up hashes
        HashA = {}
        HashB = {}
        # Populate HashA and HashB:
        if EnableShift and len(self.PeptideA.Modifications) > 0:
            self.HashPeaksWithShift(HashA, self.SpectrumA, self.PeptideA, VerboseFlag)
        else:
            self.HashPeaks(HashA, self.SpectrumA)
        if EnableShift and len(self.PeptideB.Modifications) > 0:
            self.HashPeaksWithShift(HashB, self.SpectrumB, self.PeptideB, VerboseFlag)
        else:
            self.HashPeaks(HashB, self.SpectrumB)
        #Do Dot Product
        MaxBins = max(HashA.keys())
        MaxBins = max(MaxBins, max(HashB.keys()))
        DotProduct = 0
        SumSqA = 0
        SumSqB = 0
        for I in range(MaxBins):
            A = HashA.get(I, 0)
            B = HashB.get(I, 0)
            if VerboseFlag and (A or B):
                print "%s\t%s\t%s\t%s\t%s\t"%(I, I/self.BinMultiplier, A, B, A*B)
            DotProduct += A * B
            SumSqA += A * A
            SumSqB += B * B
        #print "Dot Product is %f"%DotProduct
        #print "sqrt thing is %f"%math.sqrt(SumSqA * SumSqB)
        return DotProduct / math.sqrt(SumSqA * SumSqB)
    def HashPeaksWithShift(self, Hash, Spectrum, Peptide, VerboseFlag = 0):
        """
        Takes a peptide, spectrum, and hashtable
        and puts values into the hash so that it can be dotproducted (now there's a verb
        from a noun!)
        The caveat is that we shift peaks by the mass of the PTM.
        WARNING: Currently written for only ONE PTM per peptide
        """
        ModIndices = Peptide.Modifications.keys()
        ModList = Peptide.Modifications[ModIndices[0]]
        FirstMod = ModList[0]
        ModMass = FirstMod.Mass
        ModIndex = ModIndices[0] + 1
        PeptideLength = len(Peptide.Aminos)
        for Peak in Spectrum.Peaks:
            if self.MaxIntensityRank != None and Peak.IntensityRank > self.MaxIntensityRank:
                continue
            Bin = int(round(Peak.Mass * self.BinMultiplier)) # default
            if Peak.IonType:
                if Peak.IonType.Name in ("b", "b-h2o", "b-nh3", "b-h2o-h2o", "b-h2o-nh3", "a"):
                    if Peak.PeptideIndex >= ModIndex:
                        Bin = int(round((Peak.Mass - ModMass) * self.BinMultiplier))
                        if VerboseFlag:
                            print "Peak at %s is %s %s, shift left to %s"%(Peak.Mass, Peak.IonType.Name, Peak.PeptideIndex, Bin)
                if Peak.IonType.Name in ("b2",):
                    if Peak.PeptideIndex >= ModIndex:
                        Bin = int(round((Peak.Mass - ModMass/2.0) * self.BinMultiplier))
                        if VerboseFlag:
                            print "Peak at %s is %s %s, shift halfleft to %s"%(Peak.Mass, Peak.IonType.Name, Peak.PeptideIndex, Bin)
                if Peak.IonType.Name in ("y", "y-h2o", "y-nh3", "y-h2o-nh3", "y-h2o-h2o"):
                    if (PeptideLength - Peak.PeptideIndex) < ModIndex:
                        Bin = int(round((Peak.Mass - ModMass) * self.BinMultiplier))
                        if VerboseFlag:
                            print "Peak at %s is %s %s, shift right to %s"%(Peak.Mass, Peak.IonType.Name, Peak.PeptideIndex, Bin)
                if Peak.IonType.Name in ("y2",):
                    if (PeptideLength - Peak.PeptideIndex) < ModIndex:
                        Bin = int(round((Peak.Mass - ModMass/2.0) * self.BinMultiplier))
                        if VerboseFlag:
                            print "Peak at %s is %s %s, shift halfright to %s"%(Peak.Mass, Peak.IonType.Name, Peak.PeptideIndex, Bin)
            Value = Peak.Intensity
            if self.HashByRank:
                Value = 10.0 / (10 + Peak.IntensityRank)                        
            Hash[Bin] = Hash.get(Bin, 0) + Value
    def HashPeaks(self, Hash, Spectrum):
        """
        Hashes Peak intensities into integer bins
        """
        for Peak in Spectrum.Peaks:
            if self.MaxIntensityRank != None and Peak.IntensityRank > self.MaxIntensityRank:
                continue # only deal with the good peaks
            Bin = int(round(Peak.Mass * self.BinMultiplier))
            Value = Peak.Intensity
            if self.HashByRank:
                Value = 10.0 / (10 + Peak.IntensityRank)
            Hash[Bin] = Hash.get(Bin, 0) + Value
    def GetSharedPeakCount(self, PeakWeight, RankWeight, SkewMultiplier = 1.0,
            PeakCountDivisor = 40, EnableShift = 1, VerboseFlag = 0):
        """
        Measure the shared peak count between two spectra.
        Iterate over the top N peaks of spectrum A (where N = (ParentMass / 100)*5).
        If the peak is present (modulo epsilon) as one of the top N peaks in spectrum B,
        then receive score PeakWeight + RankWeight / IntensityRank.  The sum of these
        scores is then scaled relative to the maximum attainable score.
        """
        SkewMultipliers = []
        for X in range(5):
            SkewMultipliers.append(SkewMultiplier ** X)
        self.BinMultiplier = 1.0
        SortedPeaksA = []
        for Peak in self.SpectrumA.Peaks:
            SortedPeaksA.append((Peak.Intensity, Peak))
        SortedPeaksA.sort()
        SortedPeaksA.reverse()
        N = int(round(self.SpectrumA.ParentMass / float(PeakCountDivisor)))
        #print "N = %s/%s = %s"%(self.SpectrumA.ParentMass, PeakCountDivisor, N)
        self.SpectrumB.RankPeaksByIntensity()
        # Populate HashedPeaks[Bin] = list of peaks in SpectrumB
        # which fall into Bin.  Only peaks
        # with rank <= N are hashed.
        HashedPeaks = {}
        PeptideLength = len(self.PeptideB.Aminos)
        ModIndices = self.PeptideB.Modifications.keys()
        if len(ModIndices):
            ModIndex = ModIndices[0] + 1
            ModMass = self.PeptideB.Modifications[ModIndices[0]][0].Mass
        else:
            ModIndex = None
        for Peak in self.SpectrumB.Peaks:
            if Peak.IntensityRank > N:
                continue
            Peak.ShiftedMass = Peak.Mass
            if EnableShift and ModIndex != None and Peak.IonType:
                if Peak.IonType.Name in ("b", "b-h2o", "b-nh3", "b-h2o-h2o", "b-h2o-nh3", "a"):
                    if Peak.PeptideIndex >= ModIndex:
                        Peak.ShiftedMass -= ModMass
                if Peak.IonType.Name in ("b2",):
                    if Peak.PeptideIndex >= ModIndex:
                        Peak.ShiftedMass -= (ModMass / 2.0)
                if Peak.IonType.Name in ("y", "y-h2o", "y-nh3", "y-h2o-nh3", "y-h2o-h2o"):
                    if (PeptideLength - Peak.PeptideIndex) < ModIndex:
                        Peak.ShiftedMass -= ModMass
                if Peak.IonType.Name in ("y2",):
                    if (PeptideLength - Peak.PeptideIndex) < ModIndex:
                        Peak.ShiftedMass -= (ModMass / 2.0)
            Bin = int(round(Peak.ShiftedMass * self.BinMultiplier))
            if not HashedPeaks.has_key(Bin):
                HashedPeaks[Bin] = []
            HashedPeaks[Bin].append(Peak)
##        ########################
##        # Debug peak hashing:
##        Keys = HashedPeaks.keys()
##        Keys.sort()
##        for Key in Keys:
##            Str = "%s: "%Key
##            for Peak in HashedPeaks[Key]:
##                Str += "(#%d %.1f, %.2f)"%(Peak.IntensityRank, Peak.Mass, Peak.Intensity)
##            print Str
##        ########################        
        OverallScore = 0
        MaxScore = 0
        PeptideLength = len(self.PeptideA.Aminos)
        ModIndices = self.PeptideA.Modifications.keys()
        if len(ModIndices):
            ModIndex = ModIndices[0] + 1
            ModMass = self.PeptideA.Modifications[ModIndices[0]][0].Mass
        else:
            ModIndex = None
        for PeakIndex in range(min(N, len(SortedPeaksA))):
            Peak = SortedPeaksA[PeakIndex][1]
            Peak.ShiftedMass = Peak.Mass
            if EnableShift and ModIndex != None and Peak.IonType:
                if Peak.IonType.Name in ("b", "b-h2o", "b-nh3", "b-h2o-h2o", "b-h2o-nh3", "a"):
                    if Peak.PeptideIndex >= ModIndex:
                        Peak.ShiftedMass -= ModMass
                if Peak.IonType.Name in ("b2",):
                    if Peak.PeptideIndex >= ModIndex:
                        Peak.ShiftedMass -= (ModMass / 2.0)
                if Peak.IonType.Name in ("y", "y-h2o", "y-nh3", "y-h2o-nh3", "y-h2o-h2o"):
                    if (PeptideLength - Peak.PeptideIndex) < ModIndex:
                        Peak.ShiftedMass -= ModMass
                if Peak.IonType.Name in ("y2",):
                    if (PeptideLength - Peak.PeptideIndex) < ModIndex:
                        Peak.ShiftedMass -= (ModMass / 2.0)
            Bin = int(round(Peak.ShiftedMass * self.BinMultiplier))
            BestPeak = None
            BestScore = 0
            for NearBin in (Bin - 1, Bin, Bin + 1):
                PeakList = HashedPeaks.get(NearBin, [])
                for PeakB in PeakList:
                    Skew = abs(Peak.ShiftedMass - PeakB.ShiftedMass)
                    SkewDeciDaltons = int(round(abs(Skew) * 10))
                    if SkewDeciDaltons >= 5:
                        continue
                    #Score = PeakWeight / float(RankWeight + Peak.IntensityRank + PeakB.IntensityRank)
                    Score = PeakWeight*10 + RankWeight*10 / (10.0 + Peak.IntensityRank)
                    Score *= SkewMultipliers[SkewDeciDaltons]
                    if Score > BestScore:
                        BestScore = Score
                        BestPeak = PeakB
                        BestPeakScoreMultiplier = SkewMultipliers[SkewDeciDaltons]
            if VerboseFlag:
                Str = "PeakA #%d %.1f (bin %d):"%(Peak.IntensityRank, Peak.Mass, Bin)
                if Peak.IonType:
                    Str += " (%s %s)"%(Peak.IonType.Name, Peak.PeptideIndex)
                print Str
                if BestPeak:
                    Str = "  Best near peak #%d %.1f ==> %s"%(BestPeak.IntensityRank, BestPeak.Mass, BestScore)
                    if BestPeak.IonType:
                        Str += " (%s %s)"%(BestPeak.IonType.Name, BestPeak.PeptideIndex)
                    print Str
            OverallScore += BestScore
            MaxScore += PeakWeight*10 + RankWeight*10 / (10.0 + Peak.IntensityRank)
            #MaxScore += PeakWeight / float(RankWeight + Peak.IntensityRank + Peak.IntensityRank)
        return OverallScore / float(MaxScore)
    def ComputeCorrelationCoefficient(self, BinMultiplier = 1.0, MaxIntensityRank = 50, EnableShift = 1, VerboseFlag = 0, HashByRank = 0):
        """
        Compute similarity between two spectra by computing the
        correlation coefficient of the binned intensities.
        """
        self.BinMultiplier = BinMultiplier
        self.MaxIntensityRank = MaxIntensityRank
        self.HashByRank = HashByRank
        # set up hashes
        HashA = {}
        HashB = {}
        # Populate HashA and HashB:
        if EnableShift and len(self.PeptideA.Modifications) > 0:
            self.HashPeaksWithShift(HashA, self.SpectrumA, self.PeptideA, VerboseFlag)
        else:
            self.HashPeaks(HashA, self.SpectrumA)
        if EnableShift and len(self.PeptideB.Modifications) > 0:
            self.HashPeaksWithShift(HashB, self.SpectrumB, self.PeptideB, VerboseFlag)
        else:
            self.HashPeaks(HashB, self.SpectrumB)
        MinBin = min(HashA.keys())
        MinBin = min(MinBin, min(HashB.keys()))
        MaxBin = max(HashA.keys())
        MaxBin = max(MaxBin, max(HashB.keys()))
        TotalA = 0
        TotalB = 0
        BinCount = MaxBin - MinBin + 1
        for Bin in range(MinBin, MaxBin + 1):
            A = HashA.get(Bin, 0)
            B = HashB.get(Bin, 0)
            TotalA += A
            TotalB += B
        MeanA = TotalA / float(BinCount)
        MeanB = TotalB / float(BinCount)
        if VerboseFlag:
            print "MeanA %s over %s bins"%(MeanA, BinCount)
            print "MeanB %s over %s bins"%(MeanB, BinCount)
        SigmaSumA = 0
        SigmaSumB = 0
        for Bin in range(MinBin, MaxBin + 1):
            A = HashA.get(Bin, 0)
            B = HashB.get(Bin, 0)
            SigmaSumA += (A - MeanA)**2
            SigmaSumB += (B - MeanB)**2
        VarianceA = SigmaSumA / float(BinCount)
        SigmaA = math.sqrt(VarianceA)
        VarianceB = SigmaSumB / float(BinCount)
        SigmaB = math.sqrt(VarianceB)
        if VerboseFlag:
            print "A has variance %s stddev %s"%(VarianceA, SigmaA)
            print "B has variance %s stddev %s"%(VarianceB, SigmaB)
        CovarianceSum = 0
        for Bin in range(MinBin, MaxBin + 1):
            A = HashA.get(Bin, 0)
            B = HashB.get(Bin, 0)
            CovarianceSum += (A - MeanA) * (B - MeanB)
        Covariance = CovarianceSum / float(BinCount - 1)
        CorrelationCoefficient = Covariance / (SigmaA * SigmaB)
        if VerboseFlag:
            print "Covariance %s, corr.coeff %s"%(Covariance, CorrelationCoefficient)
        return CorrelationCoefficient

def Test():
    FileName = "..\mzxml\Dicty-HeavyCells-11.mzXML"
    FileHandle = open(FileName,"rb")
    FileHandle.seek(60145805)
    S1 = MSSpectrum.SpectrumClass()
    S1.ReadPeaksFromFile(FileHandle,FileName) #it also sets the PrecursorMZ
    Annotation1 = "MKRKLLK"

    FileName = "..\mzxml\Dicty-HeavyCells-13.mzXML"
    FileHandle = open(FileName,"rb")
    FileHandle.seek(59307113)
    S2 = MSSpectrum.SpectrumClass()
    S2.ReadPeaksFromFile(FileHandle,FileName) #it also sets the PrecursorMZ
    Annotation2 = "MKRKLLK"

    FileName = "..\mzxml\Dicty-HeavyCells-12.mzXML"
    FileHandle = open(FileName,"rb")
    FileHandle.seek(87683754)
    S3 = MSSpectrum.SpectrumClass()
    S3.ReadPeaksFromFile(FileHandle,FileName) #it also sets the PrecursorMZ
    Annotation3 = "MKIFIIK"

    FileName = "..\mzxml\Dicty-HeavyCells-05.mzXML"
    FileHandle = open(FileName,"rb")
    FileHandle.seek(102201432)
    S5 = MSSpectrum.SpectrumClass()
    S5.ReadPeaksFromFile(FileHandle,FileName) #it also sets the PrecursorMZ
    #S5.FilterPeaks()
    Annotation5 = "NWNGQPVGVPQGQYANMNYAR"   

    FileName = "..\mzxml\Dicty-HeavyCells-12.mzXML"
    FileHandle = open(FileName,"rb")
    FileHandle.seek(112303085)
    S6 = MSSpectrum.SpectrumClass()
    S6.ReadPeaksFromFile(FileHandle,FileName) #it also sets the PrecursorMZ
    Annotation6 = "NWNGQPVGVPQGQYANMNYAR+14"   

    FileName = "..\mzxml\Dicty-HeavyCells-12.mzXML"
    FileHandle = open(FileName,"rb")
    FileHandle.seek(111782847)
    S7 = MSSpectrum.SpectrumClass()
    S7.ReadPeaksFromFile(FileHandle,FileName) #it also sets the PrecursorMZ
    Annotation7 = "NWNGQPVGVPQGQYANMNYAR"   

    Simm = SpectralSimilarity(S5,S5,Annotation5,Annotation5)
    Simm.SpectralAlignment()

if __name__ == "__main__":
    # Command-line arguments: Two spectra, then two annotations.
    if len(sys.argv)<5:
        print "Not enough arguments:", sys.argv
        sys.exit(-1)
    Comparator = SpectralSimilarity(sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4])
    Comparator.LabelPeaks()
    Result = Comparator.GetSharedPeakCount(1, 0, 0.9, PeakCountDivisor = 20, VerboseFlag = 1)
    print "Shared 1 0 0.8:", Result
    print "\n\n"
    Result = Comparator.GetSharedPeakCount(0, 1, 0.9, PeakCountDivisor = 20, VerboseFlag = 1)
    print "Shared 0 1 0.8:", Result
    print "\n\n"
    
##    Result = Comparator.GetSharedPeakCount(0, 1, PeakCountDivisor = 20, VerboseFlag = 1)
##    print "Shared 0 1 1.0:", Result
##    sys.exit(0)
##    print "Shared 0 1 0.66:", Comparator.GetSharedPeakCount(0, 1, 0.66, PeakCountDivisor = 5)
##    print "Shared 0 1 0.66:", Comparator.GetSharedPeakCount(0, 1, 0.66, PeakCountDivisor = 50)
##    
##    Result = Comparator.DotProduct(VerboseFlag = 1)
##    print "Dot product similarity score 1.0:", Result
##    Result = Comparator.DotProduct(2.0, VerboseFlag = 1)
##    print "Dot product similarity score 2.0:", Result
    Result = Comparator.DotProduct(0.5, VerboseFlag = 1, HashByRank = 1)
    print "Dot product similarity score 0.5:", Result
##    print "Shared 1 0:", Comparator.GetSharedPeakCount(1, 0)
##    print "Shared 0 1:", Comparator.GetSharedPeakCount(0, 1)
##    
##    print "Shared 1 1:", Comparator.GetSharedPeakCount(1, 1)
##    print "Cov/corr:", Comparator.ComputeCorrelationCoefficient()
    Command = "label.py \"%s\" %s"%(sys.argv[1], sys.argv[3])
    os.system(Command)
    Command = "label.py \"%s\" %s"%(sys.argv[2], sys.argv[4])
    os.system(Command)



#Title:          ProteinGrouper.py
#Author:         Stephen Tanner, Samuel Payne, Natalie Castellana, Pavel Pevzner, Vineet Bafna
#Created:        2005
# Copyright 2007,2008,2009 The Regents of the University of California
# All Rights Reserved
#
# Permission to use, copy, modify and distribute any part of this
# program for educational, research and non-profit purposes, by non-profit
# institutions only, without fee, and without a written agreement is hereby
# granted, provided that the above copyright notice, this paragraph and
# the following three paragraphs appear in all copies.
#
# Those desiring to incorporate this work into commercial
# products or use for commercial purposes should contact the Technology
# Transfer & Intellectual Property Services, University of California,
# San Diego, 9500 Gilman Drive, Mail Code 0910, La Jolla, CA 92093-0910,
# Ph: (858) 534-5815, FAX: (858) 534-7345, E-MAIL:invent@ucsd.edu.
#
# IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
# FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
# INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
# OF SUCH DAMAGE.
#
# THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
# OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
# ENHANCEMENTS, OR MODIFICATIONS.  THE UNIVERSITY OF CALIFORNIA MAKES NO
# REPRESENTATIONS AND EXTENDS NO WARRANTIES OF ANY KIND, EITHER IMPLIED OR
# EXPRESS, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, OR THAT THE USE OF
# THE SOFTWARE WILL NOT INFRINGE ANY PATENT, TRADEMARK OR OTHER RIGHTS.

#Updated 1-3-2012 to allow column header based lookup (NEC)

import os
import sys
import ResultsParser
import TrieUtils
import Utils
import getopt

UsageInfo = """ProteinGrouper.py version 2012.01.03
ProteinGrouper updates the 'Protein' field for Inspect annotations, replacing
the single protein name with a '!' delimited list of protein names.  For each 
Inspect results file specified, a new file is created with the updated 'Protein'
field.  
[REQUIRED]
-r [File or Dir] File or directory containing Inspect annotations
-t [File] Trie file used to search spectra, assumes an index file exists of the same name
-w [Dir] Directory where updated Inspect annotations are written

[OPTIONAL]:
-p Assign peptides to a parsimonious set of proteins.  
-a Assign peptides to a parsimonious set of proteins.  This protein will
appear first in the list of proteins contained by the peptide.
"""
DELIM = "!"

class ProteinGrouper(ResultsParser.ResultsParser):
    def __init__(self):
        ResultsParser.ResultsParser.__init__(self)
        self.Columns = ResultsParser.Columns()
        Utils.Initialize()
        self.TrieFiles = []
        self.IndexFiles = []
        self.DoParsimony = 0
        self.DoParsimonyAndGroup = 0
        self.TUtils = TrieUtils.TrieUtils()


        self.Peptide2ProteinID = {} #Peptide sequence -> (TrieIndex,ProteinIDS)
        self.Peptide2SpectralCount = {} #Peptide sequence -> Spectral Count
        self.Protein2Peptides = {} # (TrieIndex,ProteinID) -> Peptide sequences
        #self.ProteinNames = {} #(TrieIndex,ProteinID) -> ProteinName

        #Populated only if parsimony
        self.SelectedProteins = {}
        self.HeaderLine = None
        

    def Main(self):

        #Load peptides
        for FileName in self.InputFiles:
            print "Loading peptides from %s"%FileName
            self.LoadPeptides(FileName)
        if self.DoParsimony == 1 or self.DoParsimonyAndGroup == 1:
            self.ChoosePeptides()
        for FileName in self.InputFiles:
            self.WritePeptides(FileName)
            

    #We assume that if multiple peptides are listed for a spectrum, then they appear together in the file and in decreasing order of
    #confidence
    def LoadPeptides(self,FileName):
        #print "Loading peptides!!"
        #raw_input()
        LocalDebug = 1

        RawFileName = os.path.basename(FileName)
        
        File = open(FileName,'r')
        #OutputFile = os.path.join(self.OutputDir,RawFileName)
        #OutFile = open(OutputFile ,'w')
        PeptideList = {}
        
        PrevSpectrum = None
        
        
        for Line in File:
            Line = Line.strip()
            if Line == "":
                continue
            if Line[0] == "#":
                self.HeaderLine = Line
                continue
            
            if LocalDebug:
                print Line
            
            Bits = Line.split("\t")
            ModdedPeptide = Bits[self.Columns.getIndex("Annotation")]
            Peptide = Utils.GetPeptideFromModdedName(ModdedPeptide).Aminos

            #if Peptide == "YGPLLDLPELPFPELER":
            #    LocalDebug = 1
            #else:
            #    LocalDebug = 0
            CurrSpectrum = (Bits[0],Bits[1])
            if CurrSpectrum == PrevSpectrum:
                continue
            PrevSpectrum = CurrSpectrum

            #Update the spectral count for this peptide
            if not self.Peptide2SpectralCount.has_key(Peptide):
                self.Peptide2SpectralCount[Peptide] = 0
            self.Peptide2SpectralCount[Peptide] += 1

            if LocalDebug:
                print self.Peptide2SpectralCount[Peptide]
                #raw_input()

            if self.Peptide2ProteinID.has_key(Peptide):
                if LocalDebug:
                    print "Already searched for peptide %s"%Peptide
                    raw_input()
                continue
                
            else:
                PeptideList[Peptide] = 1
                if LocalDebug:
                    print "Searching %s for the first time!"%Peptide


                #See if we've reached enough peptides to search
                if len(PeptideList.keys()) >= TrieUtils.MIN_TRIE_SEARCH_SIZE:
                    if LocalDebug:
                        print "Reached %s peptides to search"%(len(PeptideList.keys()))
                        #raw_input()
                    
                    #Loop through each trie file that we have
                    for TrieIndex in range(0,len(self.TrieFiles)):
                        if LocalDebug:
                            print "Searching trieDB: %s"%(self.TrieFiles[TrieIndex])
                        Locations = self.TUtils.GetAllLocations(PeptideList.keys(),self.TrieFiles[TrieIndex])
                        if LocalDebug:
                            print "Finished searching"
                        #For each peptide that we searched, add it's locations and proteins
                        for Pep in PeptideList.keys():
                            #if Pep == "AAGARPLTSPESLSR" or Pep == "GFFDPNTHENLTYLQLLR" or Pep == "EMAVPDVHLPDVQLPK" or Pep == "YGPLLDLPELPFPELER":
                            #    LocalDebug = 1
                            #else:
                            #    LocalDebug = 0
                            if LocalDebug:
                                print Pep
                                print "Total Locations: %s"%(len(Locations[Pep]))
                            if len(Locations[Pep]) == 0:
                                print "No locations found for %s"%Pep
                                continue
                            if not self.Peptide2ProteinID.has_key(Pep):
                                self.Peptide2ProteinID[Pep] = []
                            for (ID,Res) in Locations[Pep]:

                                if LocalDebug:
                                    print "%s appear in %s at pos %s"%(Pep,ID,Res)
                                #Get the protein name
                                #if not self.ProteinNames.has_key((TrieIndex,ID)):
                                ProteinName = self.TUtils.GetProteinName(self.IndexFiles[TrieIndex],ID)
                                    
                                #else:
                                #    ProteinName = self.ProteinNames[(TrieIndex,ID)]
                                if LocalDebug:
                                    print "Hit to protine %s"%ProteinName
                                if ProteinName[0:3] == "XXX": #Skip hits to reverse DB
                                    continue
                                if ProteinName.find(DELIM) >= 0:
                                    print "ERROR: Protein %s contains delim %s"%(ProteinName,DELIM)
                                    sys.exit(0)
                                #self.ProteinNames[(TrieIndex,ID)] = ProteinName

                                if self.Peptide2ProteinID[Pep].count((TrieIndex,ID)) == 0:
                                    self.Peptide2ProteinID[Pep].append((TrieIndex,ID))
                               
                                if not self.Protein2Peptides.has_key((TrieIndex,ID)):
                                    self.Protein2Peptides[(TrieIndex,ID)] = []
                                if self.Protein2Peptides[(TrieIndex,ID)].count(Pep) == 0:
                                    self.Protein2Peptides[(TrieIndex,ID)].append(Pep)

                    PeptideList = {}
        if len(PeptideList.keys()) > 0:
            if LocalDebug:
                print "Reached %s peptides to search"%(len(PeptideList.keys()))
                
            for TrieIndex in range(0,len(self.TrieFiles)):
                Locations = self.TUtils.GetAllLocations(PeptideList.keys(),self.TrieFiles[TrieIndex])
                    
                for Pep in PeptideList.keys():
                    if LocalDebug:
                        print Pep
                        
                    if len(Locations[Pep]) == 0:
                        print "No locations found for %s"%Pep
                        continue
                    if not self.Peptide2ProteinID.has_key(Pep):
                        self.Peptide2ProteinID[Pep] = []

                    for (ID,Res) in Locations[Pep]:

                        #Get the protein name
                        #if not self.ProteinNames.has_key((TrieIndex,ID)):
                        ProteinName = self.TUtils.GetProteinName(self.IndexFiles[TrieIndex],ID)

                        #else:
                        #    ProteinName = self.ProteinNames[(TrieIndex,ID)]

                        if ProteinName[0:3] == "XXX": #Skip hits to reverse DB
                            continue
                        if ProteinName.find(DELIM) >= 0:
                            print "ERROR: Protein %s contains delim %s"%(ProteinName,DELIM)
                            sys.exit(0)
                        #self.ProteinNames[(TrieIndex,ID)] = ProteinName

                        if self.Peptide2ProteinID[Pep].count((TrieIndex,ID)) == 0:
                            self.Peptide2ProteinID[Pep].append((TrieIndex,ID))
                        
                        if not self.Protein2Peptides.has_key((TrieIndex,ID)):
                            self.Protein2Peptides[(TrieIndex,ID)] = []
                        if self.Protein2Peptides[(TrieIndex,ID)].count(Pep) == 0:
                            self.Protein2Peptides[(TrieIndex,ID)].append(Pep)
                        

                        
                   
            
        File.close()
        

    def ChoosePeptides(self):
        
        LocalDebug = 1
        if LocalDebug:
            print "Total peptides: %s"%(len(self.Peptide2ProteinID.keys()))
            print "Total protiens: %s"%(len(self.Protein2Peptides.keys()))


        ProteinCounts = {}
        for (TrieIndex,ID) in self.Protein2Peptides.keys():
            SpecCount = 0
            for Pep in self.Protein2Peptides[(TrieIndex,ID)]:
                SpecCount += self.Peptide2SpectralCount[Pep]
            ProteinCounts[(TrieIndex,ID)] = (len(self.Protein2Peptides[(TrieIndex,ID)]),SpecCount)

        self.SelectedProteins = {}
        self.FinalPeptideProteins = {} #peptide -> final protein selection
        
        while (1):
            BestCandidate = None
            BestScore = None
            BestProteinName = None
            #Find the next best protein (best = most peptides)
            for (TrieIndex,ProteinID) in ProteinCounts.keys():

                #We've already added this guy
                if self.SelectedProteins.has_key((TrieIndex,ProteinID)):
                    continue
                Score = ProteinCounts[(TrieIndex,ProteinID)]
                CurrProteinName = self.TUtils.GetProteinName(self.IndexFiles[TrieIndex],ProteinID)

                if Score > BestScore or BestScore == None or (Score == BestScore and CurrProteinName < BestProteinName):
                    BestScore = Score
                    BestCandidate = (TrieIndex,ProteinID)
                    BestProteinName = CurrProteinName
                    #print "New Best %s, score %s"%(ProteinID,BestScore)
            if not BestScore:
                break
            (PeptideCount, SpectrumCount) = BestScore
            if PeptideCount == 0:
                break
            #%%%
            ProteinName = BestProteinName
            print "Accept protein %s (%s)\n  Gets %s peptides, %s spectra"%(BestCandidate, ProteinName, PeptideCount, SpectrumCount)
            self.SelectedProteins[BestCandidate] = BestScore
            # Lay claim to all the (not-yet-claimed) peptides:
            for Peptide in self.Protein2Peptides[BestCandidate]:
                if LocalDebug:
                    print "  Grab %s spectra from peptide %s"%(self.Peptide2SpectralCount[Peptide], Peptide)
                self.FinalPeptideProteins[Peptide] = BestCandidate
                # Other proteins (if not already accepted) lose a peptide, and some spectra:
                for (OtherTrieIndex,OtherID) in self.Peptide2ProteinID[Peptide]:
                    if self.SelectedProteins.has_key((OtherTrieIndex,OtherID)):
                        continue
                    #if LocalDebug:
                    #    print "Removing spectra from other Protein %s/%s (%s)"%(OtherTrieIndex,OtherID,self.ProteinNames[(OtherTrieIndex,OtherID)])
                    (pCount,sCount) = ProteinCounts[(OtherTrieIndex,OtherID)]

                    if LocalDebug:
                        print "Old counts: %s peptides %s spectra"%(pCount,sCount)
                    self.Protein2Peptides[(OtherTrieIndex,OtherID)].remove(Peptide)
                    pCount -= 1
                    sCount -= self.Peptide2SpectralCount[Peptide]
                    if LocalDebug:
                        print "New counts: %s peptides %s spectra"%(pCount,sCount)
                    ProteinCounts[(OtherTrieIndex, OtherID)] = (pCount,sCount)
        # Sanity check - the selected proteins have peptides, the unselected proteins have 0
        for Protein in self.Protein2Peptides.keys():
            #ProteinName = self.ProteinNames[Protein]
            PeptideCount = len(self.Protein2Peptides[Protein])
            SpectrumCount = ProteinCounts.get(Protein, 0)
            if self.SelectedProteins.has_key(Protein) and PeptideCount <= 0:
                print "** Warning: Selected protein %s has %s peptides!"%(Protein, PeptideCount)
            if not self.SelectedProteins.has_key(Protein) and PeptideCount != 0:
                print "** Warning: Unelected protein %s has %s peptides!"%(Protein, PeptideCount)
            
        

    def WritePeptides(self,FileName):
        RawFileName = os.path.basename(FileName)
        InputFile = open(FileName,'r')
        OutputFile = os.path.join(self.OutputDir,RawFileName)
        OutFile = open(OutputFile ,'w')
        
        MissCount = 0
        LineCount = 0
        for Line in InputFile:
            Line = Line.strip()
            if Line == "":
                continue
            if Line[0] == "#":
                OutFile.write(Line + "\n")
                continue
            Bits = Line.split("\t")
            ModdedPeptide = Bits[self.Columns.getIndex("Annotation")]
            Peptide = Utils.GetPeptideFromModdedName(ModdedPeptide).Aminos


            #See if we are doing parsimony
            if self.DoParsimonyAndGroup == 1:
                if not self.Peptide2ProteinID.has_key(Peptide) or len(self.Peptide2ProteinID[Peptide]) == 0:
                    print "ERROR: Peptide %s of %s has no locations!!!"%(Peptide,Line)
                    MissCount += 1
                    continue

                if not self.FinalPeptideProteins.has_key(Peptide):
                   print "ERROR: Peptide %s of %s has no selected protein!!!"%(Peptide,Line)
                   print "Formerly found in:"
                   for (TrieIndex,ID) in self.Peptide2ProteinID[Peptide]:
                       print "(%s,%s)"%(TrieIndex,ID)
                   continue 
                Protein = self.FinalPeptideProteins[Peptide]
                (TrieIndex,ProtID) = Protein
                
                #Add other proteins

                

                Locations = self.Peptide2ProteinID[Peptide]
                LocStr = self.TUtils.GetProteinName(self.IndexFiles[TrieIndex],ProtID)
                for Prot in Locations:
                    if Prot != Protein:
                        (TIndex,PID) = Prot
                        LocStr += DELIM + self.TUtils.GetProteinName(self.IndexFiles[TIndex],PID)
                Bits[self.Columns.getIndex("Protein")] = LocStr

            elif self.DoParsimony == 1:
                if not self.FinalPeptideProteins.has_key(Peptide):
                   print "ERROR: Peptide %s of %s has no selected protein!!!"%(Peptide,Line)
                   MissCount += 1
                   continue 
                Protein = self.FinalPeptideProteins[Peptide]
                (TrieIndex,ProtID) = Protein
                Bits[self.Columns.getIndex("Protein")] = self.TUtils.GetProteinName(self.IndexFiles[TrieIndex],ProtID) 
                Bits[self.Columns.getIndex("RecordNumber")] = str(Protein[1])
            else:
                if not self.Peptide2ProteinID.has_key(Peptide) or len(self.Peptide2ProteinID[Peptide]) == 0:
                    print "ERROR: Peptide %s of %s has no locations!!!"%(Peptide,Line)
                    MissCount += 1
                    continue

                Locations = self.Peptide2ProteinID[Peptide]
                LocStr = ""
                for Prot in Locations:
                    (TIndex,PD) = Prot
                    LocStr += self.TUtils.GetProteinName(self.IndexFiles[TIndex],PID) + DELIM
                LocStr = LocStr[0:-1*len(DELIM)]
                
                Bits[self.Columns.getIndex("Protein")] = LocStr
            Str = "\t".join(Bits)
            OutFile.write("%s\n"%Str)
            #print Str
            LineCount += 1

        print "Total peptides omitted: %s"%MissCount
        print "Wrote %s lines to %s"%(LineCount,OutputFile)
        OutFile.close()
        InputFile.close()
                    
            

    def ParseCommandLine(self,Arguments):
        (Options,Args) = getopt.getopt(Arguments,"r:w:t:pa")
        OptionsSeen = {}
        for (Option,Value) in Options:
            OptionsSeen[Option] = 1
            
            if Option == "-r":
                if not os.path.exists(Value):
                    print "ERROR: %s is not a valid file or directory"%Value
                    sys.exit(0)
                
                if not os.path.isdir(Value):
                    self.InputFiles = [Value]
                
                else:
                    Files = os.listdir(Value)
                    self.InputFiles = []
                    for FileName in Files:
                        self.InputFiles.append(os.path.join(Value,FileName))

            elif Option == "-w":
                if not os.path.exists(Value):
                    os.makedirs(Value)
                self.OutputDir = Value
            elif Option == "-t":
                if not os.path.isfile(Value):
                    print "ERROR: %s is not a valid database file"%Value
                    sys.exit(0)

                IndexFileName = os.path.splitext(Value)[0] + ".index"
                if not os.path.isfile(IndexFileName):
                    print "ERROR: Unable to find index file %s for trie file %s"%(IndexFileName,Value)
                    sys.ext(0)
                self.TrieFiles.append(Value)
                self.IndexFiles.append(IndexFileName)
            elif Option == "-p":
                self.DoParsimony = 1
            elif Option == "-a":
                self.DoParsimonyAndGroup = 1

            else:
                print "ERROR %s is not a valid argument"%Option
        
        if not OptionsSeen.has_key("-r") or not OptionsSeen.has_key("-w") or not OptionsSeen.has_key("-t"):
            print "ERROR: Missing arguments"
            print UsageInfo
            sys.exit(0)


if __name__ == "__main__":
    Grouper = ProteinGrouper()
    Grouper.ParseCommandLine(sys.argv[1:])
    Grouper.Main()



# -*- coding: UTF-8 -*-
"""
Automatized configuration and execution of Inspect peptide identification for
a list of spectrum files and a list of reference proteomes. Specifications of
posttranslational modifications can either be directly passed by the user or
assigned to the dataset by its filename (if dataset group is already known).

@author: Anke Penzlin, June 2013
"""

import re
import os
import sys
import optparse

#from InspectParser_FDRcut import parseInspect
from simulation_based_similarity import prepDB, run_inspect

def runInspect_config(spectra,
					DBs,
					spec_path,
					db_path="/data/NG4/anke/proteome/",
					inspect_dir = "/home/franziska/bin/Inspect/",
					conf = "/data/NG4/anke/Inspect/config_Inspect_py.txt",
					user_mods=""):
	"""
	run Inspect for each pair of spectrum dataset and proteome database using
	modifications according to the dataset in the configuration file.
	"""

	rngDB = range(len(DBs)) # 3 for example
	rngSpc = range(len(spectra)) # 2 for example
	simMat = [ [0 for i in rngDB] for j in rngSpc ] # initializing output: [[0, 0, 0], [0, 0, 0]]

	for i in rngSpc:
		specs = spec_path+spectra[i]+".mgf"
		for j in rngDB:
			db_j = db_path+DBs[j]+"_decoy.trie"

			# create trie if necessary (.trie and .index created simultaneously)
			if not os.path.exists(db_j):
				# a prepare decoyDB input for Inspect (def)
				go_on = prepDB(db_path+DBs[j]+"_decoy.fasta", path=inspect_dir) # Convert a protein database into concatenated format.
				if not go_on: return
			inspect_out = specs[:-4] +"_"+DBs[j]+"_InspectOut.txt" # -4 to remove file extension: .mgf


			# prepare configfile for InspecT
			conf_out = open(conf,'w')
			conf_out.write("spectra,"+specs+"\n")
			conf_out.write("instrument,FT-Hybrid\n")
			conf_out.write("protease,Trypsin\n")
			conf_out.write("DB,"+db_j+"\n")
			if not user_mods == "":
				conf_out.write(user_mods)
			elif re.search("Lacto_131",spectra[i]):
				conf_out.write("mod,46.0916,C,fix\n")
				conf_out.write("mod,15.994915,M\n")
				conf_out.write("# iTraq\n")
				conf_out.write("mod,144.1544,K,fix\n")
				conf_out.write("mod,144.1544,*,nterminal\n")
				print "modifications according to acc. nr. 13105-13162"
				sys.stdout.flush()
			elif re.search("Shigelladys",spectra[i]):
				conf_out.write("mod,46.0916,C,fix\n")
				conf_out.write("mod,15.994915,M\n")
				print "modifications according to http://www.biomedcentral.com/1471-2180/11/147#sec2"
				sys.stdout.flush()
			else:
				conf_out.write("# Protecting group on cysteine\n")
				conf_out.write("mod,57.021464,C,fix\n")
				if re.search("Bacicer_113",spectra[i]):
					conf_out.write("mod,15.994915,M\n")
					print "modifications according to acc. nr. 11339-11362"
					sys.stdout.flush()
				elif re.search("Bacisub_175",spectra[i]):
					conf_out.write("mod,15.994915,M\n")
					conf_out.write("mod,119.1423,C\n")
					conf_out.write("mod,396.37,C\n")
					print "modifications according to acc. nr. 17516-17659"
					sys.stdout.flush()
				elif re.search("Ecoli_12",spectra[i]):
					conf_out.write("mod,32,M,opt\n")
					print "modifications according to acc. nr. 12189-12199"
					sys.stdout.flush()
				elif re.search("Strepyo_1923",spectra[i]):
					conf_out.write("mod,15.994915,M\n")
					conf_out.write("mod,79.9799,STY\n")
					print "modifications according to acc. nr. 19230/19231"
					sys.stdout.flush()
				elif re.search("CPXV_",spectra[i]):
					conf_out.write("mod,15.994915,M\n")#oxidation
					conf_out.write("mod,42.010565,*,nterminal\n")#acetylation
					print "modifications according to standard configuration (for pox)"
				elif re.search("MSSim",spectra[i]):
					conf_out.write("mod,0.984016,NQ\n")
					conf_out.write("mod,15.994915,M\n")
					print "modifications according to (simulation) standard configuration"
					sys.stdout.flush()
				else:
					# conf_out.write("mod,15.994915,M\n")#oxidation
					# conf_out.write("mod,42.010565,*,nterminal\n")#acetylation
					#conf_out.write("mod,0.984016,NQ\n")
					print "modifications according to (unspecified) standard configuration"
					sys.stdout.flush()
			conf_out.write("mods,2\n")
			if re.search("Shigelladys",spectra[i]):
				conf_out.write("PMTolerance,1.4\n")
				conf_out.write("IonTolerance,0.5\n")
				conf_out.write("MultiCharge,3\n")
			else:
				conf_out.write("ParentPPM,10\n")
				conf_out.write("IonTolerance,0.8\n")
			conf_out.close()

			# run Inspect: match spectra against database
			if re.search( "Ecoli_12", spectra[i] ):
				AA_file = inspect_dir + "AminoAcidMasses_15N.txt"
				if os.path.exists(AA_file):
					run_inspect(conf, inspect_out, inspect_dir, "-a "+AA_file)
					print "amino acid masses according to 15N (because of special e.coli data set)."
					sys.stdout.flush()
				else:
					run_inspect(conf, inspect_out, inspect_dir)
					print "WARNING: file containing amino acid masses according to 15N not found!\nDatabase search using usual file disregarding special e.coli data set)."
					sys.stdout.flush()
			else:
				run_inspect(conf, inspect_out, inspect_dir)
	# 		# evaluate results from Inspect to calculate an FDR-matrix
	# 		simMat[i][j] = parseInspect(inspect_out)[2]

	# for line in simMat:
	# 	print line


if __name__=="__main__":
	usage = """%prog SPECTRA DB_LIST -s SPEC_DIR -d DB_DIR

					run InsPecT (for multiple spectrum datasets and references),
					using known modification options (assigned by filename),
					and calculate FDR-corrected identification counts from InsPecT output.

					SPECTRA: ','-separated spectrum-filenames (mgf-format) without file extension
					DB_LIST: ','-separated proteome-filenames (fasta-format) without file extension

					Using the easy mode (--easy) to have a quick understand of this function.
					"""

	# configure the parser
	optparser = optparse.OptionParser(usage=usage)
	optparser.add_option('-s', '--specdir', type='string', dest='spec_dir', default="/data/NG4/anke/spectra/", help='directory of specFiles (absolute path!). [default: %default]')
	optparser.add_option('-d', '--dbdir', type='string', dest='db_dir', default="/data/NG4/anke/proteome/", help='directory of proteinDBs (absolute path!). [default: %default]')
	optparser.add_option('-c', '--configfile', type='string', dest='config', default="/data/NG4/anke/Inspect/config_Inspect_py.txt", help='a txt-file for Inspect configuration, will be written. [default: %default]')
	optparser.add_option('-m', '--mods', type='string', dest='mods', default="", help='a string containing all modifications in question, modification choice by filename if "". [default: %default]')
	optparser.add_option('-i', '--inspect_dir', type='string', dest='ins_dir', default="/home/franziska/bin/Inspect", help='directory of Inspect.exe. [default: %default]')
	optparser.add_option('-e', '--easy', type='string', dest='easy', default=None, help='For the beginner, setting the value to True to run this script. Use true or True to activate.')

	# parse options and arguments
	options, args = optparser.parse_args()

	if not options.easy:
		if len(args) == 2:
			spectra = args[0].split(',')
			db_list = args[1].split(',')
		else:
			optparser.print_help()
			sys.exit(1)

		'''db_path = options.db_dir
		spec_path = options.spec_dir
		configfile = options.config # Inspect configuration file
		mods = options.mods
		inspect_dir = options.ins_dir'''
		runInspect_config(spectra=spectra, DBs=db_list, spec_path=options.spec_dir, db_path=options.db_dir, inspect_dir=options.ins_dir, conf=options.config, user_mods=options.mods)
	# Easy mode
	if (options.easy).lower() == 'true':
		runInspect_config(
			spectra=['example'],
			DBs=['species1', 'species2'],
			spec_path='../data/spectra/',
			db_path='../data/reference/',
			inspect_dir='./inspect/',
			conf='./config_files/config_Inspect_py.txt',
			user_mods='')



#Title:          Global.py
#Author:         Stephen Tanner, Samuel Payne, Natalie Castellana, Pavel Pevzner, Vineet Bafna
#Created:        2005
# Copyright 2007,2008,2009 The Regents of the University of California
# All Rights Reserved
#
# Permission to use, copy, modify and distribute any part of this
# program for educational, research and non-profit purposes, by non-profit
# institutions only, without fee, and without a written agreement is hereby
# granted, provided that the above copyright notice, this paragraph and
# the following three paragraphs appear in all copies.
#
# Those desiring to incorporate this work into commercial
# products or use for commercial purposes should contact the Technology
# Transfer & Intellectual Property Services, University of California,
# San Diego, 9500 Gilman Drive, Mail Code 0910, La Jolla, CA 92093-0910,
# Ph: (858) 534-5815, FAX: (858) 534-7345, E-MAIL:invent@ucsd.edu.
#
# IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
# FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
# INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
# OF SUCH DAMAGE.
#
# THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
# OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
# ENHANCEMENTS, OR MODIFICATIONS.  THE UNIVERSITY OF CALIFORNIA MAKES NO
# REPRESENTATIONS AND EXTENDS NO WARRANTIES OF ANY KIND, EITHER IMPLIED OR
# EXPRESS, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, OR THAT THE USE OF
# THE SOFTWARE WILL NOT INFRINGE ANY PATENT, TRADEMARK OR OTHER RIGHTS.


"""
Global variables for mass-spec analysis
"""

IsotopeWeights = {}

# Keys are ion type names, values are the corresponding ion instances
AllIonDict = {}

# Masses for amino acids (keys: 1-letter peptide abbreviations, like "G" or "D")
AminoMass = {}
AminoMassRight = {}

# List of all amino acid (left) masses:
AminoMasses = []

# Dictionary of post-translational modifications.  Keys are modification
# names (in lower-case).
PTMods = {}

# Truncated, 3- or 4-character keys:
PTModByShortName = {}
PTModList = []

AminoAcids = {} # key: single-letter abbreviation ("A" -> Alanine)
FixedMods = {"C":57.0518} # The protecting group on C is enabled, by default!

#List of  ModificationTypeObject (see Utils.py) the user defines as invivo or invitro
InVivoMods = []
InVitroMods = []

